@@ -24,6 +24,10 @@
 #include <gtk/gtk.h>
 #ifdef USE_GNOME
 #include <gnome.h>
+#else
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #endif
 
 #include "i18n.h"
@@ -140,7 +144,6 @@ cheat_sheet_dialog (void)
     gtk_box_pack_start (GTK_BOX (hbox), thing, FALSE, FALSE, 0);
     gtk_widget_show (thing);
 
-
     text = gtk_text_new (NULL, NULL);
     gtk_text_set_editable (GTK_TEXT (text), FALSE);
     gtk_text_set_word_wrap (GTK_TEXT (text), FALSE);
