@@ -902,15 +902,18 @@ GtkWidget* envelope_box_new(const gchar *label)
 
     // Here comes the graphical stuff
 #ifdef USE_GNOME
-    if(gui_settings.gui_use_aa_canvas) { // gnome-libs-1.0.12 says aa canvas is buggy
-	gtk_widget_push_visual (gdk_rgb_get_visual ());
-	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-	canvas = gnome_canvas_new_aa ();
-    } else {
+// gnome-libs-1.0 explicitly states that AA canvas is buggy
+// gnome-libs-1.2 doesn't change anything
+// gnome-libs-1.4 doesn't even work (program hangs in infinite loop)
+//    if(gui_settings.gui_use_aa_canvas) { // gnome-libs-1.0.12 says aa canvas is buggy
+//	gtk_widget_push_visual (gdk_rgb_get_visual ());
+//	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+//	canvas = gnome_canvas_new_aa ();
+//    } else {
 	gtk_widget_push_visual (gdk_imlib_get_visual ());
 	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 	canvas = gnome_canvas_new ();
-    }
+//    }
     e->canvas = GNOME_CANVAS(canvas);
     gtk_widget_pop_colormap ();
     gtk_widget_pop_visual ();
