@@ -54,16 +54,18 @@ gui_prefs gui_settings = {
 
     1,
     0,
+    0,
 
     0,
     1,
 
+    TRUE,/* sharps or flats */
+    FALSE,
+    
     50,
     40,
     500000,
 
-    1,
-
     -666,
     0,
     0,
@@ -101,9 +103,13 @@ static gui_subs_slider prefs_trackerfreq_slider = {
 static void
 gui_settings_close_requested (void)
 {
+/* I needed to change this code because I use some object from gui settings
+   window in the other place and these objects must not be destroyed. (yaliaev)
     gtk_container_remove(GTK_CONTAINER(ts_box), trackersettings);
     gtk_widget_destroy(configwindow);
-    configwindow = NULL;
+*/
+    gdk_window_hide(configwindow->window);
+//    configwindow = NULL;
 }
 
 static void
@@ -139,12 +145,6 @@ gui_settings_upcase_toggled (GtkWidget *widget)
     }
 }
 
-static void
-gui_settings_advance_cursor_toggled (GtkWidget *widget)
-{
-    gui_settings.advance_cursor_in_fx_columns = GTK_TOGGLE_BUTTON(widget)->active;
-}
-
 static void
 gui_settings_asyncedit_toggled (GtkWidget *widget)
 {
@@ -179,12 +179,6 @@ gui_settings_save_geometry_toggled (GtkWidget *widget)
     }
 }
 
-static void
-gui_settings_non_empty_patterns_toggled (GtkWidget *widget)
-{
-    gui_settings.save_all_patterns = GTK_TOGGLE_BUTTON(widget)->active;
-}
-
 static void
 gui_settings_scopebufsize_changed (GtkSpinButton *spin)
 {
@@ -194,14 +188,13 @@ gui_settings_scopebufsize_changed (GtkSpinButton *spin)
 }
 
 static void
-gui_settings_highlight_rows_toggled (GtkWidget *widget)
+gui_settings_bh_toggled (GtkWidget *widget)
 {
-    gui_settings.highlight_rows = GTK_TOGGLE_BUTTON(widget)->active;
-
-    tracker_redraw(tracker);
+    gui_settings.bh = GTK_TOGGLE_BUTTON(widget)->active;
+    tracker_redraw (tracker);
 }
 
-static void
+void
 gui_settings_highlight_rows_changed (GtkSpinButton *spin)
 {
     int n = gtk_spin_button_get_value_as_int(spin);
@@ -211,7 +204,7 @@ gui_settings_highlight_rows_changed (GtkSpinButton *spin)
 	tracker_redraw(tracker);
 }
 
-static void
+void
 gui_settings_highlight_rows_minor_changed (GtkSpinButton *spin)
 {
     int n = gtk_spin_button_get_value_as_int(spin);
@@ -305,10 +298,12 @@ gui_settings_dialog (void)
     gchar stmp[5];
 
     if(configwindow != NULL) {
+	if (!gdk_window_is_visible(configwindow->window))
+	    gdk_window_show(configwindow->window);
 	gdk_window_raise(configwindow->window);
 	return;
     }
-    
+
 #ifdef USE_GNOME
     configwindow = gnome_app_new("SoundTracker", _("GUI Configuration"));
 #else
@@ -361,13 +356,6 @@ gui_settings_dialog (void)
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_upcase_toggled), NULL);
 
-    thing = gtk_check_button_new_with_label(_("Horizontal effect column editing"));
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.advance_cursor_in_fx_columns);
-    gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-    gtk_signal_connect(GTK_OBJECT(thing), "toggled",
-		       GTK_SIGNAL_FUNC(gui_settings_advance_cursor_toggled), NULL);
-
     thing = gtk_check_button_new_with_label(_("Asynchronous (IT-style) pattern editing"));
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.asynchronous_editing);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
@@ -396,6 +384,13 @@ gui_settings_dialog (void)
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_save_geometry_toggled), NULL);
 
+    thing = gtk_check_button_new_with_label(_("Use note name B instead of H"));
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.bh);
+    gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
+    gtk_widget_show(thing);
+    gtk_signal_connect(GTK_OBJECT(thing), "toggled",
+		       GTK_SIGNAL_FUNC(gui_settings_bh_toggled), NULL);
+
     gui_subs_set_slider_value(&prefs_scopesfreq_slider, gui_settings.scopes_update_freq);
     gui_subs_set_slider_value(&prefs_trackerfreq_slider, gui_settings.tracker_update_freq);
 
@@ -418,30 +413,6 @@ gui_settings_dialog (void)
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
 
-    box1 = gtk_hbox_new(FALSE, 4);
-    gtk_widget_show(box1);
-    gtk_box_pack_start(GTK_BOX(vbox1), box1, FALSE, TRUE, 0);
-
-    thing = gtk_check_button_new_with_label(_("Highlight rows:"));
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.highlight_rows);
-    gtk_signal_connect(GTK_OBJECT(thing), "toggled",
-		       GTK_SIGNAL_FUNC(gui_settings_highlight_rows_toggled), NULL);
-    gtk_box_pack_start(GTK_BOX(box1), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-    add_empty_hbox(box1);
-    thing = extspinbutton_new(GTK_ADJUSTMENT(gtk_adjustment_new((double)gui_settings.highlight_rows_n, 1, 32, 1, 2, 0.0)), 0, 0);
-    gtk_box_pack_start(GTK_BOX(box1), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(thing), 0);
-    gtk_signal_connect(GTK_OBJECT(thing), "changed",
-		       GTK_SIGNAL_FUNC(gui_settings_highlight_rows_changed), NULL);
-    thing = extspinbutton_new(GTK_ADJUSTMENT(gtk_adjustment_new((double)gui_settings.highlight_rows_minor_n, 1, 16, 1, 2, 0.0)), 0, 0);
-    gtk_box_pack_start(GTK_BOX(box1), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(thing), 0);
-    gtk_signal_connect(GTK_OBJECT(thing), "changed",
-		       GTK_SIGNAL_FUNC(gui_settings_highlight_rows_minor_changed), NULL);
-
     /* Track line format */
     box1 = gtk_hbox_new(FALSE, 4);
     gtk_widget_show(box1);
@@ -494,17 +465,6 @@ gui_settings_dialog (void)
     gtk_signal_connect(GTK_OBJECT(thing), "changed",
         GTK_SIGNAL_FUNC(gui_settings_tracker_line_effect_modified), 0);
 
-    thing = gtk_hseparator_new();
-    gtk_widget_show(thing);
-    gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
-
-    thing = gtk_check_button_new_with_label(_("`Save XM' saves all non-empty patterns"));
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.save_all_patterns);
-    gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-    gtk_signal_connect(GTK_OBJECT(thing), "toggled",
-		       GTK_SIGNAL_FUNC(gui_settings_non_empty_patterns_toggled), NULL);
-
     thing = gtk_vseparator_new();
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(mainhbox), thing, FALSE, TRUE, 0);
@@ -533,7 +493,7 @@ gui_settings_dialog (void)
 #ifdef USE_GNOME
     thing = gnome_stock_button (GNOME_STOCK_BUTTON_CLOSE);
 #else
-    thing = gtk_button_new_with_label (_ ("Close"));
+    thing = gtk_button_new_with_label (_("Close"));
 #endif
     GTK_WIDGET_SET_FLAGS(thing, GTK_CAN_DEFAULT);
     gtk_window_set_default(GTK_WINDOW(configwindow), thing);
@@ -575,7 +535,9 @@ gui_settings_load_config (void)
 	prefs_get_int(f, "tracker-update-frequency", &gui_settings.tracker_update_freq);
 	prefs_get_int(f, "scopes-update-frequency", &gui_settings.scopes_update_freq);
 	prefs_get_int(f, "scopes-buffer-size", &gui_settings.scopes_buffer_size);
-	prefs_get_int(f, "save-all-non-empty-patterns", &gui_settings.save_all_patterns);	
+  	prefs_get_int(f, "sharp", &gui_settings.sharp);
+  	prefs_get_int(f, "bh", &gui_settings.bh);
+
 	prefs_close(f);
     }
 
@@ -589,6 +551,8 @@ gui_settings_load_config (void)
 	prefs_get_string(f, "savesmpl-path", gui_settings.savesmpl_path);
 	prefs_get_string(f, "loadinstr-path", gui_settings.loadinstr_path);
 	prefs_get_string(f, "saveinstr-path", gui_settings.saveinstr_path);
+	prefs_get_string(f, "loadpat-path", gui_settings.loadpat_path);
+	prefs_get_string(f, "savepat-path", gui_settings.savepat_path);
 
 	prefs_get_string(f, "rm-path", gui_settings.rm_path);
 	prefs_get_string(f, "unzip-path", gui_settings.unzip_path);
@@ -596,6 +560,8 @@ gui_settings_load_config (void)
 	prefs_get_string(f, "gz-path", gui_settings.gz_path);
 	prefs_get_string(f, "bz2-path", gui_settings.bz2_path);
 
+	prefs_get_int(f, "gui-disable-splash", &gui_settings.gui_disable_splash);
+
 	prefs_close(f);
     }
 }
@@ -640,7 +606,8 @@ gui_settings_save_config (void)
     prefs_put_int(f, "tracker-update-frequency", gui_settings.tracker_update_freq);
     prefs_put_int(f, "scopes-update-frequency", gui_settings.scopes_update_freq);
     prefs_put_int(f, "scopes-buffer-size", gui_settings.scopes_buffer_size);
-    prefs_put_int(f, "save-all-non-empty-patterns", gui_settings.save_all_patterns);
+    prefs_put_int(f, "sharp", gui_settings.sharp);
+    prefs_put_int(f, "bh", gui_settings.bh);
 
     prefs_close(f);
 }
@@ -662,6 +629,8 @@ gui_settings_save_config_always (void)
     prefs_put_string(f, "savesmpl-path", gui_settings.savesmpl_path);
     prefs_put_string(f, "loadinstr-path", gui_settings.loadinstr_path);
     prefs_put_string(f, "saveinstr-path", gui_settings.saveinstr_path);
+    prefs_put_string(f, "loadpat-path", gui_settings.loadpat_path);
+    prefs_put_string(f, "savepat-path", gui_settings.savepat_path);
 
     prefs_put_string(f, "rm-path", gui_settings.rm_path);
     prefs_put_string(f, "unzip-path", gui_settings.unzip_path);
@@ -669,5 +638,7 @@ gui_settings_save_config_always (void)
     prefs_put_string(f, "gz-path", gui_settings.gz_path);
     prefs_put_string(f, "bz2-path", gui_settings.bz2_path);
 
+    prefs_put_int(f, "gui-disable-splash", gui_settings.gui_disable_splash);
+
     prefs_close(f);
 }
