@@ -22,6 +22,7 @@
 #include <config.h>
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
@@ -42,6 +43,7 @@
 #include "i18n.h"
 #include "gui.h"
 #include "gui-subs.h"
+#include "gui-settings.h"
 #include "xm.h"
 #include "st-subs.h"
 #include "audio.h"
@@ -68,6 +70,7 @@ int notebook_current_page = NOTEBOOK_PAGE_FILE;
 GtkWidget *editing_toggle;
 GtkWidget *gui_curins_name, *gui_cursmpl_name;
 GtkWidget *mainwindow;
+GtkWidget *alt[2], *arrow[2];
 ScopeGroup *scopegroup;
 
 static GtkWidget *gui_splash_window = NULL;
@@ -83,14 +86,53 @@ static GtkWidget *mainwindow_upper_hbox, *mainwindow_second_hbox;
 static GtkWidget *notebook;
 static GtkWidget *spin_editpat, *spin_patlen, *spin_numchans;
 static GtkWidget *cursmpl_spin;
+static GtkWidget *pbutton;
 static GtkAdjustment *adj_amplification, *adj_pitchbend;
 static GtkWidget *spin_jump, *curins_spin, *spin_octave;
+static GtkWidget *toggle_lock_editpat;
 static Playlist *playlist;
 
 guint statusbar_context_id;
 GtkWidget *status_bar;
 GtkWidget *st_clock;
 
+struct f_n_l
+{
+    FILE *file;
+    int length;
+};
+
+struct measure
+{
+    const gchar *title;
+    gint major;
+    gint minor;
+};
+
+static struct measure measure_msr[] = {
+    {"2/2", 16, 8},
+    {"3/2", 24, 8},
+    {"4/2", 32, 8},
+    {"2/4", 8,  4},
+    {"3/4", 12, 4},
+    {"4/4", 16, 4},
+    {"5/4", 20, 4},
+    {"6/4", 24, 4},
+    {"7/4", 28, 4},
+    {"3/8", 6, 2},
+    {"4/8", 8, 2},
+    {"5/8", 10, 2},
+    {"6/8", 12, 2},
+    {"9/8", 18, 2},
+    {"12/8", 24, 2},
+    {NULL}
+};
+
+#define MAXMEASURE (sizeof(measure_msr) / sizeof(struct measure) - 1)
+
+static GtkWidget *measurewindow = NULL;
+static gint measure_chosen;
+    
 static void gui_tempo_changed (int value);
 static void gui_bpm_changed (int value);
 
@@ -138,6 +180,177 @@ static void offset_current_sample(int offset);
 
 static void gui_auto_switch_page (void);
 
+static void
+gui_highlight_rows_toggled (GtkWidget *widget)
+{
+    gui_settings.highlight_rows = GTK_TOGGLE_BUTTON(widget)->active;
+
+    tracker_redraw(tracker);
+}
+
+void
+gui_accidentals_clicked (GtkWidget *widget, gpointer data)
+{
+    gui_settings.sharp = !gui_settings.sharp;
+    gtk_widget_hide(alt[gui_settings.sharp ? 1 : 0]);
+    gtk_widget_show(alt[gui_settings.sharp ? 0 : 1]);
+    tracker_redraw(tracker);
+}
+
+void
+gui_direction_clicked (GtkWidget *widget, gpointer data)
+{
+    gui_settings.advance_cursor_in_fx_columns = !gui_settings.advance_cursor_in_fx_columns;
+    gtk_widget_hide(arrow[gui_settings.advance_cursor_in_fx_columns ? 0 : 1]);
+    gtk_widget_show(arrow[gui_settings.advance_cursor_in_fx_columns ? 1 : 0]);
+}
+
+static void
+measure_close_requested (void)
+{
+#ifndef USE_GNOME
+    gtk_widget_hide(measurewindow);
+#endif
+/* to make keyboard working immediately after closing the dialog */
+    gtk_widget_grab_focus(pbutton);
+}
+
+static void
+measure_dialog (gint x, gint y)
+{
+    GtkObject *adj;
+    GtkWidget *mainbox, *thing, *vbox;
+#ifndef USE_GNOME
+    GtkWidget *button;
+#endif
+    static GtkWidget *majspin;
+
+    if(measurewindow != NULL) {
+	gtk_widget_set_uposition(measurewindow, x, y);
+	gtk_widget_show(measurewindow);
+	gtk_widget_grab_focus(majspin);
+	return;
+    }
+    
+#ifdef USE_GNOME
+    measurewindow = gnome_dialog_new(_("Row highlighting configuration"),
+			GNOME_STOCK_BUTTON_CLOSE, NULL);
+    gnome_dialog_close_hides(GNOME_DIALOG(measurewindow), TRUE);
+    gnome_dialog_set_close(GNOME_DIALOG(measurewindow), TRUE);
+    gtk_signal_connect(GTK_OBJECT(measurewindow), "clicked",
+			GTK_SIGNAL_FUNC (measure_close_requested), NULL);
+    vbox = GNOME_DIALOG(measurewindow)->vbox;
+#else
+/* stolen from Gnome UI code. With Gnome life seemed so easy... (yaliaev) */
+    measurewindow = gtk_window_new(GTK_WINDOW_DIALOG);
+    gtk_window_set_title(GTK_WINDOW(measurewindow), _("Row highlighting configuration"));
+    gtk_container_border_width(GTK_CONTAINER(measurewindow), 4);
+    
+    vbox = gtk_vbox_new(FALSE, 8);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
+    gtk_container_add(GTK_CONTAINER(measurewindow), vbox);
+    gtk_widget_show(vbox);
+
+    gtk_window_set_policy (GTK_WINDOW (measurewindow), FALSE, 
+			 FALSE, FALSE);
+
+    mainbox = gtk_vbox_new(FALSE, 8);
+    gtk_box_pack_start (GTK_BOX (vbox), mainbox, 
+		      TRUE, TRUE, 4);
+		      
+    thing = gtk_hbutton_box_new ();
+
+    gtk_button_box_set_spacing (GTK_BUTTON_BOX (thing), 8);
+    
+    button = gtk_button_new_with_label(_("Close"));
+    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
+    gtk_box_pack_start (GTK_BOX (thing), button, TRUE, TRUE, 0);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			GTK_SIGNAL_FUNC (measure_close_requested), NULL);
+
+    gtk_widget_grab_default (button);
+    gtk_widget_show (button);
+
+    gtk_box_pack_end (GTK_BOX (vbox), thing, 
+		    FALSE, TRUE, 0);
+    gtk_widget_show (thing);
+
+    thing = gtk_hseparator_new ();
+    gtk_box_pack_end (GTK_BOX (vbox), thing, 
+		      FALSE, TRUE, 4);
+    gtk_widget_show (thing);
+		      
+#endif
+
+    gtk_signal_connect(GTK_OBJECT (measurewindow), "delete_event",
+			GTK_SIGNAL_FUNC (measure_close_requested), NULL);
+
+    mainbox = gtk_hbox_new(FALSE, 2);
+    
+    gtk_widget_show(mainbox);
+
+    thing = gtk_label_new(_("Highlight rows (major / minor):"));
+    gtk_box_pack_start(GTK_BOX(mainbox), thing, FALSE, TRUE, 0);
+    gtk_widget_show(thing);
+    add_empty_hbox(mainbox);
+    adj = gtk_adjustment_new((double)gui_settings.highlight_rows_n, 1, 32, 1, 2, 0.0);
+    majspin = extspinbutton_new(GTK_ADJUSTMENT(adj), 0, 0);
+    gtk_box_pack_start(GTK_BOX(mainbox), majspin, FALSE, TRUE, 0);
+    gtk_widget_show(majspin);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(majspin), 0);
+    gtk_signal_connect(GTK_OBJECT(majspin), "changed",
+		       GTK_SIGNAL_FUNC(gui_settings_highlight_rows_changed), NULL);
+    adj = gtk_adjustment_new((double)gui_settings.highlight_rows_minor_n, 1, 16, 1, 2, 0.0);
+    thing = extspinbutton_new(GTK_ADJUSTMENT(adj), 0, 0);
+    gtk_box_pack_start(GTK_BOX(mainbox), thing, FALSE, TRUE, 0);
+    gtk_widget_show(thing);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(thing), 0);
+    gtk_signal_connect(GTK_OBJECT(thing), "changed",
+		       GTK_SIGNAL_FUNC(gui_settings_highlight_rows_minor_changed), NULL);
+    gtk_widget_set_uposition(measurewindow, x, y);
+    
+    gtk_box_pack_start(GTK_BOX(vbox), mainbox, TRUE, TRUE, 0);
+    gtk_widget_show_all(measurewindow);
+    gtk_widget_grab_focus(majspin);
+}    
+
+static void
+measure_changed (GtkWidget *list, GtkWidget *child, gpointer data)
+{
+    if((measure_chosen = gtk_list_child_position(GTK_LIST(list), child)) <= (MAXMEASURE - 1)) {
+	if ((gui_settings.highlight_rows_n != measure_msr[measure_chosen].major) ||
+		(gui_settings.highlight_rows_minor_n != measure_msr[measure_chosen].minor)) {
+	    gui_settings.highlight_rows_n = measure_msr[measure_chosen].major;
+	    gui_settings.highlight_rows_minor_n = measure_msr[measure_chosen].minor;
+	    tracker_redraw(tracker);
+/* to make keyboard working immediately after chosing the measure */
+	    gtk_widget_grab_focus(pbutton);
+	}
+    }
+}
+
+static void
+popwin_hide (GtkWidget *widget, GtkWidget *thing)
+{
+    gint x, y, w, h, dx, dy;
+    
+    if (measure_chosen == MAXMEASURE){
+	gdk_window_get_geometry (thing->window, &x, &y, &w, &h, NULL);
+	gdk_window_get_root_origin (thing->window, &dx, &dy);
+	measure_dialog(x+ dx + w + 32, y + dy + h + 24);
+    }
+}
+
+static void
+gui_update_title (const gchar *filename)
+{
+    gchar *title;
+
+    title = g_strdup_printf("SoundTracker "VERSION": %s", g_basename(filename));
+    gtk_window_set_title(GTK_WINDOW(mainwindow), title);
+    g_free(title);
+}
+
 static void
 gui_mixer_play_pattern (int pattern,
 			int row,
@@ -193,6 +406,7 @@ gui_save_callback (gint reply,
 	    xm->modified = 0;
 	    gui_auto_switch_page();
 	    statusbar_update(STATUS_MODULE_SAVED, FALSE);
+	    gui_update_title ((gchar*)data);
 	} else {
 	    statusbar_update(STATUS_IDLE, FALSE);
 	}
@@ -208,6 +422,7 @@ gui_save_song_callback (gint reply,
 	if(XM_Save(xm, (gchar*)data, TRUE)) {
 	    gui_auto_switch_page();
 	    statusbar_update(STATUS_SONG_SAVED, FALSE);
+	    gui_update_title ((gchar*)data);
 	} else {
 	    statusbar_update(STATUS_IDLE, FALSE);
 	}
@@ -227,20 +442,111 @@ gui_save_wav_callback (gint reply,
 	write(audio_ctlpipe, &i, sizeof(i));
 	write(audio_ctlpipe, &l, sizeof(l));
 	write(audio_ctlpipe, data, l + 1);
-
-	gui_set_current_pattern(xm->pattern_order_table[0]);
 	wait_for_player();
     }
 }
 
+static void
+gui_shrink_callback (gint reply,
+		     gpointer data)
+{
+    if(!reply) {
+	st_shrink_pattern((XMPattern *)data);
+	gui_update_pattern_data();
+	tracker_set_pattern(tracker, NULL);
+	tracker_set_pattern(tracker, (XMPattern *)data);
+	xm_set_modified(1);
+    }
+}
+
+void
+gui_shrink_pattern ()
+{
+    XMPattern *patt = tracker->curpattern;
+
+    if(st_check_if_odd_are_not_empty(patt)) {
+	gnome_app_ok_cancel_modal(GNOME_APP(mainwindow),
+	    _("Odd pattern rows contain data which will be lost after shrinking.\n"
+	      "Do you want to continue anyway?"),
+	    gui_shrink_callback,
+	    patt);
+    } else {
+	gui_shrink_callback(0, patt);
+    }
+}
+
+static void
+gui_expand_callback (gint reply, gpointer data)
+{
+    if(!reply) {
+	st_expand_pattern((XMPattern *)data);
+	gui_update_pattern_data();
+	tracker_set_pattern(tracker, NULL);
+	tracker_set_pattern(tracker, (XMPattern *)data);
+	xm_set_modified(1);
+    }
+}
+
+void
+gui_expand_pattern ()
+{
+    XMPattern *patt = tracker->curpattern;
+
+    if(patt->length > 128) {
+	gnome_app_ok_cancel_modal(GNOME_APP(mainwindow),
+	    _("The pattern is too long for expanding.\n"
+	      "Some data at the end of the pattern will be lost."
+	      "Do you want to continue anyway?"),
+	    gui_expand_callback,
+	    patt);
+    } else {
+	gui_expand_callback(0, patt);
+    }
+}
+
+static void
+gui_pattern_length_correct (gint reply, gpointer data)
+{
+    XMPattern *patt = tracker->curpattern;
+    struct f_n_l *ddata = (struct f_n_l*) data;
+    int length = ddata->length;
+    FILE *f = ddata->file;
+
+    switch (reply) {
+    case 0: /* Yes! */
+	st_set_pattern_length (patt, length);
+	gui_update_pattern_data ();
+    case 1: /* No! */
+	if (xm_xp_load (f, length, patt, xm)) {
+	    tracker_set_pattern (tracker, NULL);
+	    tracker_set_pattern (tracker, patt);
+	    xm_set_modified(1);
+	}
+    case 2: /* Cancel, do nothing */
+    default:
+	break;
+    }
+    fclose (f);
+}
+
 static void
 file_selected (GtkWidget *w,
 	       GtkFileSelection *fs)
 {
+    static    struct f_n_p fnp; /* we need static here 'cause we pass these */
+    static    struct f_n_l fnl; /* structures to callback functions */
+    int length;
+
     gchar *fn = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
 
     gtk_widget_hide(GTK_WIDGET(fs));
 
+    if(!file_selection_is_valid(fn)) {
+	/* No file was actually selected. */
+	gnome_error_dialog(_("No file selected."));
+	return;
+    }
+
     if(fs == GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_MOD])) {
 	file_selection_save_path(fn, gui_settings.loadmod_path);
 	if(xm->modified) {
@@ -264,6 +570,7 @@ file_selected (GtkWidget *w,
 				      fn);
 	} else {
 	    gui_save_callback(0, fn);
+		fileops_refresh_list(fs, FALSE);
 	}
     } else if(fs == GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_SONG_AS_XM])) {
 	FILE *f = fopen(fn, "r");
@@ -278,8 +585,9 @@ file_selected (GtkWidget *w,
 				      fn);
 	} else {
 	    gui_save_song_callback(0, fn);
+		fileops_refresh_list(fs, FALSE);
 	}
-    } else {
+    } else if(fs == GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_MOD_AS_WAV])) {
 	FILE *f = fopen(fn, "r");
 
 	file_selection_save_path(fn, gui_settings.savemodaswav_path);
@@ -292,7 +600,50 @@ file_selected (GtkWidget *w,
 				      fn);
 	} else {
 	    gui_save_wav_callback(0, fn);
+		fileops_refresh_list(fs, FALSE);
 	}
+    } else if(fs == GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_PATTERN])) {
+	FILE *f = fopen (fn, "r");
+	
+	file_selection_save_path(fn, gui_settings.savepat_path);
+	
+	fnp.name = fn;
+	fnp.pattern = tracker->curpattern;
+	fnp.xm = xm;
+	if(f != NULL) {
+	    fclose(f);
+		gnome_app_ok_cancel_modal(GNOME_APP(mainwindow),
+                    			_("Are you sure you want to overwrite the file?"),
+                    			xm_xp_save,
+                                 	&fnp);
+	    } else {
+    		xm_xp_save (0, &fnp);
+			fileops_refresh_list(fs, FALSE);
+	    }
+    } else if(fs == GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_PATTERN])) {
+	XMPattern *patt = tracker->curpattern;
+	FILE *f = fopen (fn, "r");
+
+	file_selection_save_path(fn, gui_settings.loadpat_path);
+	
+	if (!f) gnome_error_dialog (_("Error when opening pattern file!"));
+	else { if (xm_xp_load_header (f, &length)) {
+	    if (length == patt->length) {
+		if (xm_xp_load (f, length, patt, xm)) {
+		    tracker_set_pattern (tracker, NULL);
+		    tracker_set_pattern (tracker, patt);
+		    xm_set_modified(1);
+		}
+		fclose (f);
+	    } else {
+		fnl.file = f;
+		fnl.length = length;
+		gui_yes_no_cancel_modal (GTK_WIDGET(mainwindow),
+                                	_("The length of the pattern being loaded doesn't match with that of current pattern in module.\nDo you want to change the current pattern length?"),
+                            		gui_pattern_length_correct,
+                                	&fnl);
+	    }
+	} else fclose (f); }
     }
 }
 
@@ -352,6 +703,7 @@ gui_handle_standard_keys (int shift,
 			  guint32 keyval)
 {
     gboolean handled = FALSE, b;
+    int currpos;
 
     switch (keyval) {
     case GDK_F1 ... GDK_F7:
@@ -386,6 +738,13 @@ gui_handle_standard_keys (int shift,
 	    /* previous pattern */
 	    offset_current_pattern(shift ? -10 : -1);
 	    handled = TRUE;
+	} else if(shift){
+	    /* previous position */
+	    currpos = playlist_get_position (playlist);
+	    if((--currpos) >= 0 ) {
+		playlist_set_position (playlist, currpos);
+		handled = TRUE;
+	    }
 	}
 	break;
     case GDK_Right:
@@ -397,7 +756,14 @@ gui_handle_standard_keys (int shift,
 	    /* next pattern */
 	    offset_current_pattern(shift ? 10 : 1);
 	    handled = TRUE;
-	}
+	} else if(shift){
+	    /* next position */
+	    currpos = playlist_get_position (playlist);
+	    if ((++currpos) < xm->song_length ) {
+		playlist_set_position (playlist, currpos);
+		handled = TRUE;
+	    }
+        }
 	break;
     case GDK_Up:
 	if(ctrl) {
@@ -434,23 +800,26 @@ gui_handle_standard_keys (int shift,
             break;
 	b = GUI_ENABLED;
 	gui_play_stop();
-    if(notebook_current_page != NOTEBOOK_PAGE_SAMPLE_EDITOR && notebook_current_page != NOTEBOOK_PAGE_INSTRUMENT_EDITOR) {
-		if(b) {
-        	    /* toggle editing mode (only if we haven't been in playing mode) */
-	            gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(editing_toggle), !GUI_EDITING);
+	if(notebook_current_page != NOTEBOOK_PAGE_SAMPLE_EDITOR
+	   && notebook_current_page != NOTEBOOK_PAGE_INSTRUMENT_EDITOR) {
+	    if(b) {
+		/* toggle editing mode (only if we haven't been in playing mode) */
+		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(editing_toggle), !GUI_EDITING);
+ 		tracker_redraw(tracker);
     	        if (GTK_TOGGLE_BUTTON(editing_toggle)->active)
-					show_editmode_status();
-	            else
-					statusbar_update(STATUS_IDLE, FALSE);
-        }
-    }
+		    show_editmode_status();
+		else
+		    statusbar_update(STATUS_IDLE, FALSE);
+	    }
+	}
 	handled = TRUE;
 	break;
     case GDK_Escape:
         if(ctrl || alt || shift)
             break;
 	/* toggle editing mode, even if we're in playing mode */
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(editing_toggle), !GUI_EDITING);
+		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(editing_toggle), !GUI_EDITING);
+	    tracker_redraw(tracker);
         if (GTK_TOGGLE_BUTTON(editing_toggle)->active)
 	    show_editmode_status();
         else
@@ -474,7 +843,7 @@ keyevent (GtkWidget *widget,
 	sample_editor_handle_keys,
 	modinfo_page_handle_keys,
     };
-    gboolean pressed = (gboolean)data;
+    gboolean pressed = (gboolean)GPOINTER_TO_INT(data);
     gboolean handled = FALSE;
     gboolean entry_focus = GTK_IS_ENTRY(GTK_WINDOW(mainwindow)->focus_widget);
 
@@ -526,35 +895,42 @@ keyevent (GtkWidget *widget,
 }
 
 static void
-playlist_position_changed (Playlist *p,
-			   int newpos)
+gui_playlist_position_changed (Playlist *p,
+			       int newpos)
 {
-    if(gui_playing_mode) {
+    if(gui_playing_mode == PLAYING_SONG) {
 	// This will only be executed when the user changes the song position manually
-	gui_mixer_set_songpos(newpos);
 	event_waiter_start(audio_songpos_ew);
+	gui_mixer_set_songpos(newpos);
     } else {
-	gui_set_current_pattern(playlist_get_nth_pattern(p, newpos));
+	if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle_lock_editpat))) {
+	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_editpat),
+				      playlist_get_nth_pattern(p, newpos));
+	}
     }
 }
 
 static void
-playlist_restart_position_changed (Playlist *p,
-				   int pos)
+gui_playlist_restart_position_changed (Playlist *p,
+				       int pos)
 {
     xm->restart_position = pos;
     xm_set_modified(1);
 }
 
 static void
-playlist_entry_changed (Playlist *p,
-			int pos,
-			int pat)
+gui_playlist_entry_changed (Playlist *p,
+			    int pos,
+			    int pat)
 {
     int i;
 
     if(pos != -1) {
 	xm->pattern_order_table[pos] = pat;
+	if(pos == playlist_get_position(p)
+	    && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle_lock_editpat))) {
+	    gui_set_current_pattern(pat);
+	}
     } else {
 	for(i = 0; i < xm->song_length; i++) {
 	    xm->pattern_order_table[i] = playlist_get_nth_pattern(p, i);
@@ -565,11 +941,11 @@ playlist_entry_changed (Playlist *p,
 }
 
 static void
-playlist_song_length_changed (Playlist *p,
-			      int length)
+gui_playlist_song_length_changed (Playlist *p,
+				  int length)
 {
     xm->song_length = length;
-    playlist_entry_changed(p, -1, -1);
+    gui_playlist_entry_changed(p, -1, -1);
 }
 
 static void
@@ -577,8 +953,18 @@ gui_editpat_changed (GtkSpinButton *spin)
 {
     int n = gtk_spin_button_get_value_as_int(spin);
 
-    if(n != editing_pat)
+    if(n != editing_pat) {
 	gui_set_current_pattern(n);
+
+	/* If we are in 'playing pattern' mode and asynchronous
+	 * editing is disabled, make the audio thread jump to the new
+	 * pattern, too. I think it would be cool to have this for
+	 * 'playing song' mode, too, but then modifications in
+	 * gui_update_player_pos() will be necessary. */
+	if(gui_playing_mode == PLAYING_PATTERN && !ASYNCEDIT) {
+	    gui_mixer_set_pattern(n);
+	}
+    }
 }
 
 static void
@@ -600,7 +986,10 @@ gui_numchans_changed (GtkSpinButton *spin)
 {
     int n = gtk_spin_button_get_value_as_int(spin);
 
-    g_assert((n & 1) == 0);
+    if(n & 1) {
+	gtk_spin_button_set_value(spin, --n);
+	return;
+    }
 
     if(xm->num_channels != n) {
 	gui_play_stop();
@@ -870,8 +1259,6 @@ play_song (void)
     write(audio_ctlpipe, &i, sizeof(i));
     write(audio_ctlpipe, &sp, sizeof(sp));
     write(audio_ctlpipe, &pp, sizeof(pp));
-
-    gui_set_current_pattern(xm->pattern_order_table[sp]);
     wait_for_player();
 }
 
@@ -966,6 +1353,7 @@ gui_load_xm (const char *filename)
     } else {
 	gui_init_xm(1);
 	statusbar_update(STATUS_MODULE_LOADED, FALSE);
+	gui_update_title (filename);
     }
 }
 
@@ -1037,10 +1425,6 @@ gui_set_current_pattern (int p)
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_editpat), p);
     gui_update_pattern_data();
 
-    if(!GUI_ENABLED && !ASYNCEDIT) {
-	gui_mixer_set_pattern(p);
-    }
-
     xm_set_modified(m);
 }
 
@@ -1064,13 +1448,12 @@ offset_current_pattern (int offset)
     int nv;
 
     nv = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_editpat)) + offset;
-
     if(nv < 0)
 	nv = 0;
     else if(nv > 255)
 	nv = 255;
 
-    gui_set_current_pattern(nv);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_editpat), nv);
 }
 
 void
@@ -1116,6 +1499,15 @@ gui_get_current_jump_value (void)
 	return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_jump));
 }
 
+void
+gui_set_jump_value (int value)
+{
+    if(GUI_ENABLED || ASYNCEDIT){
+ 	g_return_if_fail (value >= 0 && value <= 16);
+ 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_jump), value);
+    }
+}
+
 int
 gui_get_current_instrument (void)
 {
@@ -1154,6 +1546,38 @@ offset_current_sample (int offset)
     gui_set_current_sample(nv);
 }
 
+static void
+gui_add_free_pattern (GtkWidget *w, Playlist *p)
+{
+    int pos = playlist_get_position(p) + 1;
+
+    if(p->length < p->max_length) {
+	int n = st_find_first_unused_and_empty_pattern(xm);
+
+	if(n != -1) {
+	    playlist_insert_pattern(p, pos, n);
+	    playlist_set_position(p, pos);
+	}
+    }
+}
+
+static void
+gui_add_free_pattern_and_copy (GtkWidget *w, Playlist *p)
+{
+    int pos = playlist_get_position(p) + 1;
+
+    if(p->length < p->max_length) {
+	int n = st_find_first_unused_and_empty_pattern(xm);
+	int c = gui_get_current_pattern();
+
+	if(n != -1) {
+	    st_copy_pattern(&xm->patterns[n], &xm->patterns[c]);
+	    playlist_insert_pattern(p, pos, n);
+	    playlist_set_position(p, pos);
+	}
+    }
+}
+
 void
 gui_get_text_entry (int length,
 		    void(*changedfunc)(),
@@ -1275,6 +1699,7 @@ gui_splash (int argc,
 
 #ifdef USE_GNOME
     gnome_init("SoundTracker", VERSION, argc, argv);
+    gdk_rgb_init();
 #else
     gtk_init(&argc, &argv);
     gdk_rgb_init();
@@ -1387,8 +1812,14 @@ int
 gui_final (int argc,
 	   char *argv[])
 {
-    GtkWidget *thing, *mainvbox, *table, *hbox, *frame, *mainvbox0;
+    GtkWidget *thing, *mainvbox, *table, *hbox, *frame, *mainvbox0, *pmw, *vbox, *list, *entry;
     GdkColormap *colormap;
+    GtkStyle *style;
+    GdkPixmap *pm;
+    GdkBitmap *mask;
+    gint i, wdth, cur, selected;
+    GList *glist;
+    gchar *other;
 #ifdef USE_GNOME
     GtkWidget *dockitem;
 #endif
@@ -1423,12 +1854,16 @@ gui_final (int argc,
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_MOD]), gui_settings.loadmod_path);
     fileops_dialogs[DIALOG_SAVE_MOD] = file_selection_create(_("Save XM..."), file_selected);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_MOD]), gui_settings.savemod_path);
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
     fileops_dialogs[DIALOG_SAVE_MOD_AS_WAV] = file_selection_create(_("Render module as WAV..."), file_selected);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_MOD_AS_WAV]), gui_settings.savemodaswav_path);
 #endif
     fileops_dialogs[DIALOG_SAVE_SONG_AS_XM] = file_selection_create(_("Save song as XM..."), file_selected);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_SONG_AS_XM]), gui_settings.savesongasxm_path);
+    fileops_dialogs[DIALOG_LOAD_PATTERN] = file_selection_create(_("Load current pattern..."), file_selected);
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_PATTERN]), gui_settings.loadpat_path);
+    fileops_dialogs[DIALOG_SAVE_PATTERN] = file_selection_create(_("Save current pattern..."), file_selected);
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_PATTERN]), gui_settings.savepat_path);
 
     mainvbox0 = gtk_vbox_new(FALSE, 0);
     gtk_container_border_width(GTK_CONTAINER(mainvbox0), 0);
@@ -1458,14 +1893,18 @@ gui_final (int argc,
 
     playlist = PLAYLIST(thing);
     gtk_signal_connect (GTK_OBJECT (playlist), "current_position_changed",
-			GTK_SIGNAL_FUNC (playlist_position_changed), NULL);
+			GTK_SIGNAL_FUNC (gui_playlist_position_changed), NULL);
     gtk_signal_connect (GTK_OBJECT (playlist), "restart_position_changed",
-			GTK_SIGNAL_FUNC (playlist_restart_position_changed), NULL);
+			GTK_SIGNAL_FUNC (gui_playlist_restart_position_changed), NULL);
     gtk_signal_connect (GTK_OBJECT (playlist), "song_length_changed",
-			GTK_SIGNAL_FUNC (playlist_song_length_changed), NULL);
+			GTK_SIGNAL_FUNC (gui_playlist_song_length_changed), NULL);
     gtk_signal_connect (GTK_OBJECT (playlist), "entry_changed",
-			GTK_SIGNAL_FUNC (playlist_entry_changed), NULL);
-
+			GTK_SIGNAL_FUNC (gui_playlist_entry_changed), NULL);
+    gtk_signal_connect (GTK_OBJECT (playlist->ifbutton), "clicked",
+			GTK_SIGNAL_FUNC (gui_add_free_pattern), playlist);
+    gtk_signal_connect (GTK_OBJECT (playlist->icbutton), "clicked",
+			GTK_SIGNAL_FUNC (gui_add_free_pattern_and_copy), playlist);
+    
     thing = gtk_vseparator_new();
     gtk_box_pack_start(GTK_BOX(mainwindow_upper_hbox), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
@@ -1478,35 +1917,83 @@ gui_final (int argc,
     gtk_box_pack_start(GTK_BOX(mainwindow_upper_hbox), table, FALSE, TRUE, 0);
     gtk_widget_show(table);
 
-    thing = gtk_button_new_with_label (_("Play Song"));
+    gtk_widget_realize(mainwindow); /* to produce the correct style... */
+    style = gtk_widget_get_style(mainwindow);
+    hbox = gtk_hbox_new(FALSE, 4);
+    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 2, 0, 1);
+    gtk_widget_show(hbox);
+
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/play.xpm");
+    pmw = gtk_pixmap_new(pm, mask);
+    pbutton = thing = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(thing), pmw);
     gtk_signal_connect (GTK_OBJECT (thing), "clicked",
 			GTK_SIGNAL_FUNC(play_song), NULL);
-    gtk_table_attach_defaults(GTK_TABLE(table), thing, 0, 1, 0, 1);
-    gtk_widget_show(thing);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE,0);
+    gui_hang_tooltip(thing, _("Play Song"));
+    gtk_widget_show_all(thing);
 
-    thing = gtk_button_new_with_label (_("Play Pattern"));
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/play_cur.xpm");
+    pmw = gtk_pixmap_new(pm, mask);
+    thing = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(thing), pmw);
     gtk_signal_connect (GTK_OBJECT (thing), "clicked",
 			GTK_SIGNAL_FUNC(play_pattern), NULL);
-    gtk_table_attach_defaults(GTK_TABLE(table), thing, 1, 2, 0, 1);
-    gtk_widget_show(thing);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE,0);
+    gui_hang_tooltip(thing, _("Play Pattern"));
+    gtk_widget_show_all(thing);
 
-    thing = gtk_button_new_with_label (_("Stop"));
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/stop.xpm");
+    pmw = gtk_pixmap_new(pm, mask);
+    thing = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(thing), pmw);
     gtk_signal_connect (GTK_OBJECT (thing), "clicked",
 			GTK_SIGNAL_FUNC(gui_play_stop), NULL);
-    gtk_table_attach_defaults(GTK_TABLE(table), thing, 0, 2, 1, 2);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE,0);		
+    gui_hang_tooltip(thing, _("Stop"));
+    gtk_widget_show_all(thing);
+    
+    add_empty_hbox(hbox);
+
+    thing = gtk_label_new(_("Pat"));
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
 
+    spin_editpat = extspinbutton_new(GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 255, 1.0, 10.0, 0.0)), 0, 0);
+    extspinbutton_disable_size_hack(EXTSPINBUTTON(spin_editpat));
+    gui_hang_tooltip(spin_editpat, _("Edited pattern"));
+    gtk_box_pack_start(GTK_BOX(hbox), spin_editpat, FALSE, TRUE, 0);
+    gtk_widget_show(spin_editpat);
+    gtk_signal_connect(GTK_OBJECT(spin_editpat), "changed",
+		       GTK_SIGNAL_FUNC(gui_editpat_changed), NULL);
+		       
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/lock.xpm");
+    pmw = gtk_pixmap_new(pm, mask);
+    toggle_lock_editpat = thing = gtk_toggle_button_new();
+    gtk_container_add(GTK_CONTAINER(thing), pmw);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE,0);		
+    gui_hang_tooltip(thing, _("When enabled, browsing the playlist does not change the edited pattern."));
+    gtk_widget_show_all(thing);
+
     thing = gui_subs_create_slider(&tempo_slider);
     gtk_table_attach_defaults(GTK_TABLE(table), thing, 0, 1, 3, 4);
 
     gtk_widget_show(thing);
 
     thing = gui_subs_create_slider(&bpm_slider);
-    gtk_table_attach_defaults(GTK_TABLE(table), thing, 0, 1, 4, 5);
+    gtk_table_attach_defaults(GTK_TABLE(table), thing, 1, 2, 3, 4);
     gtk_widget_show(thing);
 
     hbox = gtk_hbox_new(FALSE, 4);
-    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 2, 2, 3);
+    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 2, 1, 2);
     gtk_widget_show(hbox);
 
     thing = gtk_label_new(_("Number of Channels:"));
@@ -1523,27 +2010,10 @@ gui_final (int argc,
     gtk_widget_show(spin_numchans);
 
     hbox = gtk_hbox_new(FALSE, 4);
-    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, 3, 4);
+    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 2, 2, 3);
     gtk_widget_show(hbox);
 
-    thing = gtk_label_new(_("Pattern"));
-    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-
-    add_empty_hbox(hbox);
-
-    spin_editpat = extspinbutton_new(GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 255, 1.0, 10.0, 0.0)), 0, 0);
-    extspinbutton_disable_size_hack(EXTSPINBUTTON(spin_editpat));
-    gtk_box_pack_start(GTK_BOX(hbox), spin_editpat, FALSE, TRUE, 0);
-    gtk_widget_show(spin_editpat);
-    gtk_signal_connect(GTK_OBJECT(spin_editpat), "changed",
-		       GTK_SIGNAL_FUNC(gui_editpat_changed), NULL);
-    
-    hbox = gtk_hbox_new(FALSE, 4);
-    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, 4, 5);
-    gtk_widget_show(hbox);
-
-    thing = gtk_label_new(_("PatLength"));
+    thing = gtk_label_new(_("Pattern Length"));
     gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
 
@@ -1556,13 +2026,113 @@ gui_final (int argc,
 		       GTK_SIGNAL_FUNC(gui_patlen_changed), NULL);
     gtk_widget_show(spin_patlen);
 
+    hbox = gtk_hbox_new(FALSE, 4);
+    gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 2, 4, 5);
+    gtk_widget_show(hbox);
+    
+    vbox = gtk_vbox_new(FALSE, 0);
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/sharp.xpm");
+    alt[0] = gtk_pixmap_new(pm, mask);
+    gtk_box_pack_start(GTK_BOX(vbox), alt[0], FALSE, FALSE, 0);
+    
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/flat.xpm");
+    alt[1] = gtk_pixmap_new(pm, mask);
+    gtk_widget_show(alt[gui_settings.sharp ? 0 : 1]);
+    gtk_box_pack_start(GTK_BOX(vbox), alt[1], FALSE, FALSE, 0);
+    gtk_widget_show(vbox);
+    
+    thing = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(thing), vbox);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE, 0);
+    gui_hang_tooltip(thing, _("Set preferred accidental type"));
+    gtk_widget_show(thing);
+    gtk_signal_connect(GTK_OBJECT(thing), "clicked",
+		       GTK_SIGNAL_FUNC(gui_accidentals_clicked), NULL);
+
+    add_empty_hbox(hbox);
+    thing = gtk_toggle_button_new_with_label(_("Measure"));
+    gui_hang_tooltip(thing, _("Enable row highlighting"));
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing),gui_settings.highlight_rows);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE, 0);
+    gtk_signal_connect(GTK_OBJECT(thing), "toggled",
+		       GTK_SIGNAL_FUNC(gui_highlight_rows_toggled), NULL);
+    gtk_widget_show(thing);
+
+    thing = gtk_combo_new();
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE, 0);
+    gtk_widget_show(thing);
+    entry = GTK_COMBO(thing)->entry;
+    gui_hang_tooltip(entry, _("Row highlighting configuration"));
+    style = gtk_widget_get_style(entry);
+    
+    glist = NULL;
+    wdth = 0;
+    selected = -1;
+    for(i = 0; measure_msr[i].title != NULL; i++) {
+	glist = g_list_append(glist, (gpointer)measure_msr[i].title);
+	if ((cur = gdk_string_width(style->font, measure_msr[i].title)) > wdth)
+	    wdth = cur;
+	if ((measure_msr[i].major == gui_settings.highlight_rows_n) &&
+	    (measure_msr[i].minor == gui_settings.highlight_rows_minor_n))
+		selected = i;
+    }
+    if (selected == -1) selected = i;
+    other = _("Other...");
+    glist = g_list_append(glist, other);
+    if ((cur = gdk_string_width(style->font,other)) > wdth) wdth = cur;
+
+    gtk_combo_set_popdown_strings(GTK_COMBO(thing), glist);
+    list = GTK_COMBO(thing)->list;
+    gtk_list_select_item(GTK_LIST(list), selected);
+    gtk_signal_connect(GTK_OBJECT(list), "select_child",
+		       GTK_SIGNAL_FUNC(measure_changed), NULL);
+    /* the direct use of combo->popwin is not recommended, but I couldn't find another way... */
+    gtk_signal_connect(GTK_OBJECT(GTK_COMBO(thing)->popwin), "hide",
+		       GTK_SIGNAL_FUNC(popwin_hide), entry);
+    
+    gtk_widget_set_usize(entry, wdth + 5, -1);//-1 to suppress the height changing
+    gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+    
+    add_empty_hbox(hbox);
+    
+    vbox = gtk_vbox_new(FALSE, 0);
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/downarrow.xpm");
+    arrow[0] = gtk_pixmap_new(pm, mask);
+    gtk_box_pack_start(GTK_BOX(vbox), arrow[0], FALSE, FALSE, 0);
+    
+    pm = gdk_pixmap_create_from_xpm(mainwindow->window,
+	&mask, &style->bg[GTK_STATE_NORMAL],
+	PREFIX"/share/soundtracker/rightarrow.xpm");
+    arrow[1] = gtk_pixmap_new(pm, mask);
+    gtk_box_pack_start(GTK_BOX(vbox), arrow[1], FALSE, FALSE, 0);
+    gtk_widget_show(arrow[gui_settings.advance_cursor_in_fx_columns ? 1 : 0]);
+    gtk_widget_show(vbox);
+    
+    thing = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(thing), vbox);
+    gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, FALSE, 0);
+    gui_hang_tooltip(thing, _("Change effect column editing direction"));
+    gtk_widget_show(thing);
+    gtk_signal_connect(GTK_OBJECT(thing), "clicked",
+		       GTK_SIGNAL_FUNC(gui_direction_clicked), NULL);
+    
     /* Scopes Group or Instrument / Sample Listing */
 
     thing = gtk_vseparator_new();
     gtk_box_pack_start(GTK_BOX(mainwindow_upper_hbox), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
-    
+
+#ifndef NO_GDK_PIXBUF
+    scopegroup = SCOPE_GROUP(scope_group_new(gdk_pixbuf_new_from_file(PREFIX"/share/soundtracker/muted.png")));
+#else
     scopegroup = SCOPE_GROUP(scope_group_new());
+#endif
     gtk_widget_show(GTK_WIDGET(scopegroup));
     gtk_box_pack_start(GTK_BOX(mainwindow_upper_hbox), GTK_WIDGET(scopegroup), TRUE, TRUE, 0);
 
@@ -1578,6 +2148,7 @@ gui_final (int argc,
 
     adj_amplification = GTK_ADJUSTMENT(gtk_adjustment_new(7.0, 0, 8.0, 0.1, 0.1, 0.1));
     thing = gtk_vscale_new(adj_amplification);
+    gui_hang_tooltip(thing, _("Global amplification"));
     gtk_scale_set_draw_value(GTK_SCALE(thing), FALSE);
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(hbox), thing, TRUE, TRUE, 0);
@@ -1613,6 +2184,7 @@ gui_final (int argc,
 
     adj_pitchbend = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, -20.0, +20.0, 1, 1, 1));
     thing = gtk_vscale_new(adj_pitchbend);
+    gui_hang_tooltip(thing, _("Pitchbend"));
     gtk_scale_set_draw_value(GTK_SCALE(thing), FALSE);
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(hbox), thing, TRUE, TRUE, 0);
@@ -1620,6 +2192,7 @@ gui_final (int argc,
 			GTK_SIGNAL_FUNC(gui_adj_pitchbend_changed), NULL);
 
     thing = gtk_button_new_with_label("R");
+    gui_hang_tooltip(thing, _("Reset pitchbend to its normal value"));
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, TRUE, 0);
     gtk_signal_connect (GTK_OBJECT (thing), "clicked",
@@ -1631,7 +2204,7 @@ gui_final (int argc,
     gtk_box_pack_start(GTK_BOX(mainvbox), hbox, FALSE, TRUE, 0);
     gtk_widget_show(hbox);
 
-    editing_toggle = thing = gtk_check_button_new_with_label("Editing");
+    editing_toggle = thing = gtk_check_button_new_with_label(_("Editing"));
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), 0);
     gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
@@ -1640,7 +2213,7 @@ gui_final (int argc,
     gtk_box_pack_start(GTK_BOX(hbox), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
 
-    spin_octave = extspinbutton_new(GTK_ADJUSTMENT(gtk_adjustment_new(5.0, 0.0, 6.0, 1.0, 1.0, 0.0)), 0, 0);
+    spin_octave = extspinbutton_new(GTK_ADJUSTMENT(gtk_adjustment_new(3.0, 0.0, 6.0, 1.0, 1.0, 0.0)), 0, 0);
     extspinbutton_disable_size_hack(EXTSPINBUTTON(spin_octave));
     gtk_box_pack_start(GTK_BOX(hbox), spin_octave, FALSE, TRUE, 0);
     gtk_widget_show(spin_octave);
@@ -1774,15 +2347,17 @@ gui_final (int argc,
     }
 
     if(gui_splash_window) {
-	if(
+	if(!gui_settings.gui_disable_splash && (
 #ifndef NO_GDK_PIXBUF
            gui_splash_logo ||
 #endif
-           tips_dialog_show_tips) {
+           tips_dialog_show_tips)) {
 	    gdk_window_raise(gui_splash_window->window);
 //	    gtk_window_set_transient_for(GTK_WINDOW(gui_splash_window), GTK_WINDOW(mainwindow));
 // (doesn't do anything on WindowMaker)
 	    gui_splash_set_label(_("Ready."), TRUE);
+//	    gdk_window_hide(gui_splash_window->window);
+//	    gdk_window_show(gui_splash_window->window);
 #ifndef NO_GDK_PIXBUF
 	    if(gui_splash_logo) {
 		gtk_widget_add_events(gui_splash_logo_area,
