@@ -19,6 +19,12 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
+#ifndef USE_GNOME
+#include <stdlib.h>
+#endif
+
 #include "i18n.h"
 #include "instrument-editor.h"
 #include "envelope-box.h"
@@ -41,7 +47,7 @@ static GtkWidget *instrument_editor_vibtype_w[4];
 static GtkWidget *clavier;
 static GtkWidget *curnote_label;
 
-static STInstrument *current_instrument;
+static STInstrument *current_instrument, *tmp_instrument = NULL;
 
 static void
 instrument_page_volfade_changed (int value)
@@ -122,7 +128,8 @@ instrument_editor_clavierkey_enter_event (GtkWidget *widget,
 					  gint key,
 					  gpointer data)
 {
-    gtk_label_set_text(GTK_LABEL(curnote_label), notenames[key]);
+    int index = (gui_settings.sharp ? 0 : 1) + (gui_settings.bh ? 2 : 0);
+    gtk_label_set_text(GTK_LABEL(curnote_label), notenames[index][key]);
     return FALSE;
 }
 
@@ -200,6 +207,12 @@ instrument_editor_file_selected (GtkWidget *w,
 
     gtk_widget_hide(GTK_WIDGET(fs));
 
+    if(!file_selection_is_valid(fn)) {
+	/* No file was actually selected. */
+	gnome_error_dialog(_("No file selected."));
+	return;
+    }
+
     if(fs == GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_INSTRUMENT])) {
 	file_selection_save_path(fn, gui_settings.loadinstr_path);
 	instrument_editor_load_instrument(fn);
@@ -381,6 +394,29 @@ instrument_page_create (GtkNotebook *nb)
     add_empty_vbox(mainbox);
 }
 
+void
+instrument_editor_copy_instrument (STInstrument *instr)
+{
+    if (tmp_instrument == NULL)
+	tmp_instrument = calloc(1, sizeof(STInstrument));
+    st_copy_instrument(instr, tmp_instrument);
+}
+
+void
+instrument_editor_cut_instrument (STInstrument *instr)
+{
+    instrument_editor_copy_instrument(instr);
+    st_clean_instrument(instr, NULL);
+}
+
+void
+instrument_editor_paste_instrument (STInstrument *instr)
+{
+    if (tmp_instrument == NULL)
+	return;
+    st_copy_instrument(tmp_instrument, instr);
+}
+
 void
 instrument_editor_set_instrument (STInstrument *i)
 {
