@@ -48,6 +48,7 @@
 #include "keys.h"
 #include "gui-subs.h"
 #include "gui.h"
+#include "gui-settings.h"
 #include "menubar.h"
 #include "preferences.h"
 
@@ -67,8 +68,8 @@ static int cw_currentgroup = -1,
     cw_combostrings = -1;
 
 typedef struct keys_key {
-    const char *title;
-    const char *explanation;
+    char *title;
+    char *explanation;
     gboolean allkeys;
     int meaning;
 
@@ -122,10 +123,15 @@ static keys_key keys2[] = {
     { NULL }
 };
 
-static keys_key keys3[] = {
+static keys_key keys3[3+32+1] = { /* 3 fixed entries,
+				     32 dynamically generated ones (see keys_generate_channel_explanations()),
+				     1 NULL terminator */
     { "KOFF", N_("The key that inserts the special keyoff note for FastTracker modules."),
       0, KEYS_MEANING_TYPE_MAKE(KEYS_MEANING_KEYOFF), 0, 0 },
-    { NULL }
+    { "JMP+", N_("The key that increases \"jump\" value."),
+      0, KEYS_MEANING_TYPE_MAKE(KEYS_MEANING_FUNC), 0, 0 },
+    { "JMP-", N_("The key that decreases \"jump\" value."),
+      0, KEYS_MEANING_TYPE_MAKE(KEYS_MEANING_FUNC) + 1, 0, 0 },
 };
 
 typedef struct keys_group {
@@ -316,7 +322,7 @@ static void
 keys_key_group_changed (void *a,
 			void *b)
 {
-    unsigned n = (int)b;
+    unsigned n = GPOINTER_TO_INT(b);
     keys_key *kpt;
     gchar string[128];
     gchar * insertbuf[2] = { NULL, string };
@@ -765,6 +771,7 @@ keys_load_config (void)
 	gnome_warning_dialog(_("The keyboard configuration file is defective.\nPlease use the Keyboard Configuration dialog."));
 	return 0;
       break2:
+	continue; /* nonsense, but the easiest way to avoid gcc 3.2+ warnings */
     }
     r = 1;
 
@@ -873,6 +880,39 @@ keys_try_automatic_config (KeySym first,
     return 1;
 }
 
+/* This function tries to assign channel mute key combinations automatically */
+static int
+keys_ch_try_automatic_config (KeySym first,
+			     int key_offset,
+			     int count,
+			     keys_key *nk,
+			     int modifiers)
+{
+    int key;
+
+    key = find_keysym(first);
+    if(key == -1)
+	return 0;
+
+    key += key_offset * symspercode;
+
+    while(count) {
+	if(key < 0 || key >= xkeymaplen)
+	    return 0;
+
+	nk->xkeysym = xkeymap[key].xkeysym;
+	nk->xkeycode = xmin + key / symspercode;
+	nk->modifiers = modifiers;
+
+	nk += 1;
+	key += 1 * symspercode;
+
+	count--;
+    }
+
+    return 1;
+}
+
 static int
 keys_qsort_compare_func (const void *string1,
 			 const void *string2)
@@ -949,6 +989,20 @@ keys_fixup_xkeymap (void)
     }
 }
 
+/* This function fills title and explanation fields of channel selecting key combinations */
+static void
+keys_generate_channel_explanations (keys_key *array,
+				    int count)
+{
+    int i = 0;
+
+    for(i = 0; i < count; i++, array++) {
+	array->title = g_strdup_printf(_("CH%02d"), i + 1);
+	array->explanation = g_strdup_printf(_("Fast jump to channel %d"), i + 1);
+	array->meaning = KEYS_MEANING_TYPE_MAKE(KEYS_MEANING_CH) + i;
+    }
+}
+
 int
 keys_init (void)
 {
@@ -1006,12 +1060,18 @@ keys_init (void)
 
     keys_fixup_xkeymap();
     keys_make_xkeys();
+    keys_generate_channel_explanations(keys3 + 3, 32);
+    keys3[sizeof(keys3)/sizeof(keys3[0]) - 1].title = NULL;
 
     if(!keys_load_config()) {
 	if(!keys_try_automatic_config('e', -2, 12, 12, WHITE, keys1)
 	   || !keys_try_automatic_config('x', -1, 10, 0, WHITE, keys2)
 	   || !keys_try_automatic_config('2', 0, 8, 13, BLACK, keys1 + 1)
-	   || !keys_try_automatic_config('s', 0, 7, 1, BLACK, keys2 + 1)) {
+	   || !keys_try_automatic_config('s', 0, 7, 1, BLACK, keys2 + 1)
+	   || !keys_ch_try_automatic_config ('E', -2, 8, keys3 + 3, ENCODE_MODIFIERS(1, 0, 0))
+	   || !keys_ch_try_automatic_config ('S', -1, 8, keys3 + 11, ENCODE_MODIFIERS(1, 0, 0))
+	   || !keys_ch_try_automatic_config ('E', -2, 8, keys3 + 19, ENCODE_MODIFIERS(1, 1, 0))
+	   || !keys_ch_try_automatic_config ('S', -1, 8, keys3 + 27, ENCODE_MODIFIERS(1, 1, 0))) {
 	    // Automatic key configuration unsuccessful. Popup requester.
 	    gnome_warning_dialog(_("Automatic key configuration unsuccessful.\nPlease use the Keyboard Configuration dialog\n"
 				 "in the Settings menu."));
