@@ -22,6 +22,7 @@
 #include <config.h>
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -80,7 +81,13 @@ main (int argc,
 #ifdef DRIVER_SGI
 	driver_out_irix,
 #endif
-#ifndef NO_AUDIOFILE
+#ifdef DRIVER_JACK
+	driver_out_jack,
+#endif
+#ifdef DRIVER_SUN
+	driver_out_sun, driver_in_sun,
+#endif
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
 //	driver_out_file,
 #endif
 #if 0
@@ -154,6 +161,28 @@ main (int argc,
 					  &driver_in_oss);
 #endif
 
+#ifdef DRIVER_SGI
+    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
+					   &driver_out_irix);
+#endif
+
+#ifdef DRIVER_JACK
+    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
+					   &driver_out_jack);
+#endif
+
+#ifdef DRIVER_SUN
+    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
+					   &driver_out_sun);
+    drivers[DRIVER_INPUT] = g_list_append(drivers[DRIVER_INPUT],
+					  &driver_in_sun);
+#endif
+
+#ifdef _WIN32
+    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
+					   &driver_out_dsound);
+#endif
+
 #ifdef DRIVER_ALSA
     drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
 					   &driver_out_alsa);
@@ -173,17 +202,7 @@ main (int argc,
 					   &driver_out_esd);
 #endif
 
-#ifdef DRIVER_SGI
-    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
-					   &driver_out_irix);
-#endif
-
-#ifdef _WIN32
-    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
-					   &driver_out_dsound);
-#endif
-
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
 /*    drivers[DRIVER_OUTPUT] = g_list_append(drivers[DRIVER_OUTPUT],
 					   &driver_out_file);
 
@@ -211,7 +230,7 @@ main (int argc,
     if(gui_final(argc, argv)) {
 	audioconfig_load_config();
 	track_editor_load_config();
-#if defined(DRIVER_ALSA_050)
+#if defined(DRIVER_ALSA_050) || defined(DRIVER_ALSA_09x)
 	midi_load_config();
 	midi_init();
 #endif
@@ -220,6 +239,8 @@ main (int argc,
 
 	gtk_main();
 
+	gui_play_stop(); /* so that audio driver is shut down correctly. */
+
 	menubar_write_accels();
 
 	if(gui_settings.save_settings_on_exit) {
@@ -231,7 +252,7 @@ main (int argc,
 	gui_settings_save_config_always();
 	tips_dialog_save_settings();
 	track_editor_save_config();
-#if defined(DRIVER_ALSA_050)
+#if defined(DRIVER_ALSA_050) || defined(DRIVER_ALSA_09x)
 	midi_save_config();
 #endif
 	fileops_tmpclean();
