@@ -2,7 +2,7 @@
 /*
  * The Real SoundTracker - GUI (menu bar)
  *
- * Copyright (C) 1999-2002 Michael Krause
+ * Copyright (C) 1999-2003 Michael Krause
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@
 #include "instrument-editor.h"
 #include "tracker-settings.h"
 #include "midi-settings.h"
+#include "sample-editor.h"
 
 #ifdef USE_GNOME
 #include <gnome.h>
@@ -79,7 +80,7 @@ about_dialog (void)
     const gchar *authors[] = {"Michael Krause <rawstyle@soundtracker.org>", NULL};
     GtkWidget *about = gnome_about_new("SoundTracker",
 				       VERSION,
-				       "Copyright (C) 1998-2002 Michael Krause",
+				       "Copyright (C) 1998-2003 Michael Krause",
 				       authors,
 				       "Includes OpenCP player from Niklas Beisert and Tammo Hinrichs.",
 				       NULL);
@@ -94,7 +95,7 @@ about_dialog (void)
     gtk_window_position (GTK_WINDOW(about), GTK_WIN_POS_CENTER);
     gtk_window_set_title(GTK_WINDOW(about), "About SoundTracker");
 
-    label = gtk_label_new("SoundTracker " VERSION "\n\nCopyright (C) 1998-2002 Michael Krause\n<rawstyle@soundtracker.org>"
+    label = gtk_label_new("SoundTracker " VERSION "\n\nCopyright (C) 1998-2003 Michael Krause\n<rawstyle@soundtracker.org>"
 			  "\n\nIncludes OpenCP player from Niklas Beisert and Tammo Hinrichs.");
     gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(about)->vbox), 10);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, TRUE, TRUE, 10);
@@ -117,7 +118,7 @@ menubar_clear_callback (gint reply,
 			gpointer data)
 {
     if(reply == 0) {
-	if((gint)data == 0) {
+	if(GPOINTER_TO_INT(data) == 0) {
 	    gui_free_xm();
 	    gui_new_xm();
 	    xm->modified = 0;
@@ -188,6 +189,12 @@ _FUNCMACRO_(menubar_scopes_toggled)
     scope_group_enable_scopes(scopegroup, gui_settings.gui_display_scopes);
 }
 
+static void
+_FUNCMACRO_(menubar_splash_toggled)
+{
+    gui_settings.gui_disable_splash = GTK_CHECK_MENU_ITEM(widget)->active;
+}
+
 static void
 _FUNCMACRO_(menubar_mark_mode_toggled)
 {
@@ -210,11 +217,74 @@ menubar_save_settings_now (void)
     keys_save_config();
     audioconfig_save_config();
     trackersettings_write_settings();
-#if defined(DRIVER_ALSA_050) && defined(USE_GNOME)
+#if (defined(DRIVER_ALSA_050) || defined(DRIVER_ALSA_09x)) && defined(USE_GNOME)
     midi_save_config();
 #endif
 }
 
+static void
+menubar_handle_cutcopypaste (void *p, guint a)
+{
+    Tracker *t = tracker;
+    int ci = gui_get_current_instrument() - 1;
+    STInstrument *curins = &xm->instruments[ci];
+
+    switch(a){
+    case 0:		//Cut
+	switch(notebook_current_page){
+	case NOTEBOOK_PAGE_TRACKER:
+	    track_editor_cut_selection(t);
+	    break;
+	case NOTEBOOK_PAGE_INSTRUMENT_EDITOR:
+	case NOTEBOOK_PAGE_MODULE_INFO:
+	    instrument_editor_cut_instrument(curins);
+	    xm_set_modified(1);
+	    instrument_editor_update();
+	    sample_editor_update();
+	    modinfo_update_instrument(ci);
+	    break;
+	case NOTEBOOK_PAGE_SAMPLE_EDITOR:
+	    sample_editor_copy_cut_common(TRUE, TRUE);
+	    xm_set_modified(1);
+	    break;
+	}
+	break;
+    case 1:		//Copy
+	switch(notebook_current_page){
+	case NOTEBOOK_PAGE_TRACKER:
+	    track_editor_copy_selection(t);
+	    break;
+	case NOTEBOOK_PAGE_INSTRUMENT_EDITOR:
+	case NOTEBOOK_PAGE_MODULE_INFO:
+	    instrument_editor_copy_instrument(curins);
+	    break;
+	case NOTEBOOK_PAGE_SAMPLE_EDITOR:
+	    sample_editor_copy_cut_common(TRUE, FALSE);
+	    break;
+	}
+	break;
+    case 2:		//Paste
+	switch(notebook_current_page){
+	case NOTEBOOK_PAGE_TRACKER:
+	    track_editor_paste_selection(t);
+	    break;
+	case NOTEBOOK_PAGE_INSTRUMENT_EDITOR:
+	case NOTEBOOK_PAGE_MODULE_INFO:
+	    instrument_editor_paste_instrument(curins);
+	    xm_set_modified(1);
+	    instrument_editor_update();
+	    sample_editor_update();
+	    modinfo_update_instrument(ci);
+	    break;
+	case NOTEBOOK_PAGE_SAMPLE_EDITOR:
+	    sample_editor_paste_clicked();
+	    xm_set_modified(1);
+	    break;
+	}
+	break;
+    }
+}
+
 static void
 menubar_handle_edit_menu (void *p,
 			  guint a)
@@ -424,15 +494,6 @@ static GnomeUIInfo edit_selection_menu[] = {
 
     GNOMEUIINFO_SEPARATOR,
 
-    { GNOME_APP_UI_ITEM, N_("C_ut"), NULL, menubar_handle_edit_menu, (gpointer)10, NULL,
-      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 'X', GDK_CONTROL_MASK, NULL },
-    { GNOME_APP_UI_ITEM, N_("_Copy"), NULL, menubar_handle_edit_menu, (gpointer)11, NULL,
-      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 'C', GDK_CONTROL_MASK, NULL },
-    { GNOME_APP_UI_ITEM, N_("_Paste"), NULL, menubar_handle_edit_menu, (gpointer)12, NULL,
-      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, 'V', GDK_CONTROL_MASK, NULL },
-
-    GNOMEUIINFO_SEPARATOR,
-
     { GNOME_APP_UI_ITEM, N_("_Interpolate effects"), NULL, menubar_handle_edit_menu, (gpointer)8, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 'I', GDK_CONTROL_MASK, NULL },
 
@@ -452,6 +513,15 @@ static GnomeUIInfo edit_selection_menu[] = {
 };
 
 static GnomeUIInfo edit_menu[] = {
+    { GNOME_APP_UI_ITEM, N_("C_ut"), NULL, menubar_handle_cutcopypaste, (gpointer)0, NULL,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 'X', GDK_CONTROL_MASK, NULL },
+    { GNOME_APP_UI_ITEM, N_("_Copy"), NULL, menubar_handle_cutcopypaste, (gpointer)1, NULL,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 'C', GDK_CONTROL_MASK, NULL },
+    { GNOME_APP_UI_ITEM, N_("P_aste"), NULL, menubar_handle_cutcopypaste, (gpointer)2, NULL,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, 'V', GDK_CONTROL_MASK, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_TOGGLEITEM, N_("_Jazz Edit Mode"), NULL, track_editor_toggle_jazz_edit, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, ' ', GDK_SHIFT_MASK, NULL },
 
@@ -508,6 +578,20 @@ static GnomeUIInfo pattern_menu[] = {
     { GNOME_APP_UI_ITEM, N_("_Pack Patterns"), NULL, modinfo_pack_patterns, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
 
+    GNOMEUIINFO_SEPARATOR,
+
+    { GNOME_APP_UI_ITEM, N_("_Save Current Pattern"), NULL, fileops_open_dialog, (gpointer)DIALOG_SAVE_PATTERN, NULL,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS, 0, 0, NULL },
+    { GNOME_APP_UI_ITEM, N_("L_oad Pattern"), NULL, fileops_open_dialog, (gpointer)DIALOG_LOAD_PATTERN, NULL,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 0, 0, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
+    { GNOME_APP_UI_ITEM, N_("Sh_rink Current Pattern"), NULL, gui_shrink_pattern, (gpointer)0, NULL,
+      GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
+    { GNOME_APP_UI_ITEM, N_("_Expand Current Pattern"), NULL, gui_expand_pattern, (gpointer)0, NULL,
+      GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
+
     GNOMEUIINFO_END
 };
 
@@ -539,6 +623,13 @@ static GnomeUIInfo settings_tracker_menu[] = {
     { GNOME_APP_UI_ITEM, N_("_Next font"), NULL, menubar_settings_tracker_next_font, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, GDK_KP_Add, GDK_CONTROL_MASK, NULL },
 
+    GNOMEUIINFO_SEPARATOR,
+
+    { GNOME_APP_UI_ITEM, N_("Change preferred _accidental type"), 0, gui_accidentals_clicked, 0, 0,
+      GNOME_APP_PIXMAP_NONE, 0, 'S', GDK_CONTROL_MASK, NULL },
+    { GNOME_APP_UI_ITEM, N_("Change effect column editing _direction"), 0, gui_direction_clicked, 0, 0,
+      GNOME_APP_PIXMAP_NONE, 0, 'D', GDK_CONTROL_MASK, NULL },
+
     GNOMEUIINFO_END
 };
 
@@ -566,6 +657,8 @@ static GnomeUIInfo settings_menu[] = {
 
     GNOMEUIINFO_SEPARATOR,
 
+    { GNOME_APP_UI_TOGGLEITEM, N_("Disable splash screen"), 0, menubar_splash_toggled, 0, 0,
+      GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
     { GNOME_APP_UI_ITEM, N_("_Save Settings now"), NULL, menubar_save_settings_now, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL },
     { GNOME_APP_UI_TOGGLEITEM, N_("Save Settings on _Exit"), 0, menubar_save_settings_on_exit_toggled, 0, 0,
@@ -602,17 +695,18 @@ static GnomeUIInfo main_menu[] = {
 void
 menubar_init_prefs ()
 {
-#ifndef GNOME
+#ifndef USE_GNOME
     gtk_item_factory_parse_rc(prefs_get_filename("non-gnome-accels"));
 #endif
 
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[0].widget), gui_settings.gui_display_scopes);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_tracker_menu[0].widget), gui_settings.gui_use_backing_store);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[9].widget), gui_settings.save_settings_on_exit);
-#ifdef NO_AUDIOFILE
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[8].widget), gui_settings.gui_disable_splash);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[10].widget), gui_settings.save_settings_on_exit);
+#if USE_SNDFILE == 0 && defined (NO_AUDIOFILE)
     gtk_widget_set_sensitive(file_menu[SAVE_MOD_AS_WAV_POSITION].widget, FALSE);
 #endif
-#if !defined(DRIVER_ALSA_050)
+#if ! (defined(DRIVER_ALSA_050) || defined(DRIVER_ALSA_09x))
     gtk_widget_set_sensitive(settings_menu[SETTINGS_MENU_MIDI_INDEX].widget,
 			     FALSE);
 #endif	
@@ -709,19 +803,22 @@ menubar_gnome_to_gtk (GnomeUIInfo *gnometree,
 	switch(gnometree->type) {
 	case GNOME_APP_UI_SUBTREE:
 	    menubar_gtk_items[n].callback = (GtkItemFactoryCallback)gnometree->func;
-	    menubar_gtk_items[n].callback_action = (guint)gnometree->funcparam;
+	    menubar_gtk_items[n].callback_action =
+	        GPOINTER_TO_UINT(gnometree->funcparam);
 	    menubar_gtk_items[n].item_type = "<Branch>";
 	    strip_underscores(buf, buf);
 	    n = menubar_gnome_to_gtk((GnomeUIInfo*)gnometree->dummy1, n + 1, buf) - 1;
 	    break;
 	case GNOME_APP_UI_ITEM:
 	    menubar_gtk_items[n].callback = (GtkItemFactoryCallback)gnometree->func;
-	    menubar_gtk_items[n].callback_action = (guint)gnometree->funcparam;
+	    menubar_gtk_items[n].callback_action =
+	        GPOINTER_TO_UINT(gnometree->funcparam);
 	    menubar_gtk_items[n].item_type = NULL;
 	    break;
 	case GNOME_APP_UI_TOGGLEITEM:
 	    menubar_gtk_items[n].callback = (GtkItemFactoryCallback)gnometree->func;
-	    menubar_gtk_items[n].callback_action = (guint)gnometree->funcparam;
+	    menubar_gtk_items[n].callback_action =
+	        GPOINTER_TO_UINT(gnometree->funcparam);
 	    menubar_gtk_items[n].item_type = "<ToggleItem>";
 	    break;
 	case GNOME_APP_UI_SEPARATOR:
