@@ -22,12 +22,15 @@
 
 #include <config.h>
 
-#if ! defined(DRIVER_ALSA_050)
-#error "Support only ALSA API"
-#endif
+#if defined(DRIVER_ALSA_050) || defined(DRIVER_ALSA_09x)
 
+#if defined(DRIVER_ALSA_050)
 #include <sys/asoundlib.h>
+#else
+#include <alsa/asoundlib.h>
+#endif
 
 void midi_print_event(snd_seq_event_t *ev);
 
+#endif
 #endif /* _MIDI_UTILS_H */
