@@ -22,9 +22,13 @@
 
 #include <config.h>
 
-#if defined(DRIVER_ALSA_050)
+#if defined(DRIVER_ALSA_050) || defined(DRIVER_ALSA_09x)
 
+#if defined(DRIVER_ALSA_050)
 #include <sys/asoundlib.h>
+#else
+#include <alsa/asoundlib.h>
+#endif
 
 /*** Structures ***/
 
