@@ -32,6 +32,7 @@
 
 #include "mixer.h"
 #include "kb-x86-asm.h"
+#include "i18n.h"
 
 static int num_channels, mixfreq;
 static int clipflag;
@@ -330,7 +331,7 @@ kb_x86_setfreq (int channel,
     frequency /= mixfreq;
 
     c->freqw = (guint32)floor(frequency);
-    c->freqf = (guint32)((frequency - c->freqw) * pow(2,32));
+    c->freqf = (guint32)((frequency - c->freqw) * 4294967296.0 /* this is pow(2,32) */ ); 
 }
 
 static void
@@ -804,7 +805,7 @@ kb_x86_dumpstatus (st_mixer_channel_status array[])
 
 st_mixer mixer_kbfloat = {
     "kbfloat",
-    "High-quality FPU mixer, cubic interpolation, IT filters, unlimited length samples",
+    N_("High-quality FPU mixer, cubic interpolation, IT filters, unlimited length samples"),
 
     kb_x86_setnumch,
     kb_x86_updatesample,
