@@ -25,7 +25,7 @@
 
 #include <config.h>
 
-#ifndef HAVE_SYS_POLL_H
+#ifndef HAVE_POLL
 
 #include <sys/time.h>
 #include <sys/types.h>
