@@ -50,7 +50,7 @@ struct prefs_node {
     FILE *file;
 };
 
-static char *
+char *
 prefs_get_prefsdir (void)
 {
     static char xdir[128];
@@ -69,6 +69,8 @@ prefs_check_prefs_dir (void)
 
     if(stat(dir, &st) < 0) {
 	mkdir(dir, S_IRUSR | S_IWUSR | S_IXUSR);
+    strcat(dir, "/tmp");
+	mkdir(dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	error_warning(_("A directory called '.soundtracker' has been created in your\nhome directory to store configuration files.\n"));
     }
 }
