@@ -30,7 +30,9 @@
 #include <sys/ioctl.h>
 #include <dlfcn.h>
 
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE
+#include <sndfile.h>
+#elif !defined (NO_AUDIOFILE)
 #include <audiofile.h>
 #endif
 
@@ -89,7 +91,7 @@ static int sample_editor_volramp_last_values[2] = { 100, 100 };
 
 // = Load sample dialog
 
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
 enum {
     MODE_MONO = 0,
     MODE_STEREO_LEFT,
@@ -97,19 +99,29 @@ enum {
     MODE_STEREO_MIX
 };
 
-static gboolean wavload_with_audiofile;
+static gboolean wavload_through_library;
 
 static GtkWidget *wavload_dialog;
 static const char *wavload_samplename;
+
+#if USE_SNDFILE
+static SNDFILE *wavload_file;
+static SF_INFO wavinfo;
+static sf_count_t wavload_frameCount;
+#else
 static AFfilehandle wavload_file;
 static AFframecount wavload_frameCount;
+#endif
+
 static int wavload_sampleWidth, wavload_channelCount, wavload_endianness, wavload_unsignedwords;
+static long wavload_rate;
 
 static const gchar *wavload_filename;
 static GtkWidget *wavload_raw_resolution_w[2];
 static GtkWidget *wavload_raw_channels_w[2];
 static GtkWidget *wavload_raw_signed_w[2];
 static GtkWidget *wavload_raw_endian_w[2];
+static GtkWidget *wavload_raw_rate;
 
 static gboolean libaf2 = TRUE;
 #endif
@@ -139,7 +151,7 @@ static int recordedlen, sampling;
 
 static void *copybuffer = NULL;
 static int copybufferlen;
-static gboolean copybuffer_treat_as_8bit;
+static STSample copybuffer_sampleinfo;
 
 // = Realtime stuff
 
@@ -170,12 +182,14 @@ static void sample_editor_monitor_clicked(void);
 static void sample_editor_cut_clicked(void);
 static void sample_editor_remove_clicked(void);
 static void sample_editor_copy_clicked(void);
-static void sample_editor_paste_clicked(void);
+void sample_editor_paste_clicked(void);
 static void sample_editor_zoom_to_selection_clicked(void);
 
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
 static void sample_editor_load_wav(void);
 static void sample_editor_save_wav(void);
 static void sample_editor_save_region_wav(void);
+#endif
 
 static void sample_editor_open_volume_ramp_dialog(void);
 static void sample_editor_close_volume_ramp_dialog(void);
@@ -343,7 +357,7 @@ sample_editor_page_create (GtkNotebook *nb)
     gtk_widget_show(vbox);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
     fileops_dialogs[DIALOG_LOAD_SAMPLE] = file_selection_create(_("Load Sample..."), sample_editor_load_wav);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_SAMPLE]), gui_settings.loadsmpl_path);
     fileops_dialogs[DIALOG_SAVE_SAMPLE] = file_selection_create(_("Save WAV..."), sample_editor_save_wav);
@@ -357,7 +371,7 @@ sample_editor_page_create (GtkNotebook *nb)
 		       GTK_SIGNAL_FUNC(fileops_open_dialog), (void*)DIALOG_LOAD_SAMPLE);
     gtk_box_pack_start(GTK_BOX(vbox), thing, TRUE, TRUE, 0);
     gtk_widget_show(thing);
-#ifdef NO_AUDIOFILE
+#if USE_SNDFILE == 0 && defined (NO_AUDIOFILE)
     gtk_widget_set_sensitive(thing, 0);
 #endif
 
@@ -367,7 +381,7 @@ sample_editor_page_create (GtkNotebook *nb)
     gtk_box_pack_start(GTK_BOX(vbox), thing, TRUE, TRUE, 0);
     gtk_widget_show(thing);
     savebutton = thing;
-#ifdef NO_AUDIOFILE
+#if USE_SNDFILE == 0 && defined (NO_AUDIOFILE)
     gtk_widget_set_sensitive(thing, 0);
 #endif
 
@@ -377,7 +391,7 @@ sample_editor_page_create (GtkNotebook *nb)
     gtk_box_pack_start(GTK_BOX(vbox), thing, TRUE, TRUE, 0);
     gtk_widget_show(thing);
     savebutton_rgn = thing;
-#ifdef NO_AUDIOFILE
+#if USE_SNDFILE == 0 && defined (NO_AUDIOFILE)
       gtk_widget_set_sensitive(thing, 0);
 #endif
 
@@ -462,10 +476,14 @@ sample_editor_page_create (GtkNotebook *nb)
     gtk_box_pack_start(GTK_BOX(vbox), thing, TRUE, TRUE, 0);
     gtk_widget_show(thing);
 
-#if !defined(NO_AUDIOFILE) && HAVE_DLFCN_H
+#if (USE_SNDFILE || !defined (NO_AUDIOFILE)) && HAVE_DLFCN_H
     { // hack, hack
 	void *handle, *function;
+#ifdef DL_LAZY
+	handle = dlopen(NULL, DL_LAZY);
+#else
 	handle = dlopen(NULL, RTLD_NOW);
+#endif
 	function = dlsym(handle, "afSetVirtualPCMMapping");
 	if(function == NULL) {
 	    libaf2 = FALSE;
@@ -599,7 +617,7 @@ sample_editor_update (void)
 	gtk_widget_set_sensitive(se->vertical_boxes[1], TRUE);
 	gtk_widget_set_sensitive(se->vertical_boxes[2], TRUE);
 
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
 	gtk_widget_set_sensitive(savebutton, 1);
 	gtk_widget_set_sensitive(savebutton_rgn, 1);
 #endif
@@ -989,7 +1007,7 @@ sample_editor_zoom_to_selection_clicked (void)
     sample_display_set_window(sampledisplay, sampledisplay->sel_start, sampledisplay->sel_end);
 }
 
-static void
+void
 sample_editor_copy_cut_common (gboolean copy,
 			       gboolean spliceout)
 {
@@ -1014,13 +1032,13 @@ sample_editor_copy_cut_common (gboolean copy,
 	copybufferlen = cutlen;
 	copybuffer = malloc(copybufferlen * 2);
 	if(!copybuffer) {
-	    error_error("Out of memory for copybuffer.\n");
+	    error_error(_("Out of memory for copybuffer.\n"));
 	} else {
 	    memcpy(copybuffer,
 		   oldsample->sample.data + ss,
 		   cutlen * 2);
 	}
-	copybuffer_treat_as_8bit = oldsample->treat_as_8bit;
+	memcpy(&copybuffer_sampleinfo, oldsample, sizeof(STSample));
     }
 
     if(!spliceout)
@@ -1119,7 +1137,7 @@ sample_editor_init_sample (const char *samplename)
     current_sample->relnote = 0;
 }
 
-static void
+void
 sample_editor_paste_clicked (void)
 {
     gint16 *newsample;
@@ -1134,7 +1152,11 @@ sample_editor_paste_clicked (void)
 	/* pasting into empty sample */
 	sample_editor_lock_sample();
 	sample_editor_init_sample(_("<just pasted>"));
-	oldsample->treat_as_8bit = copybuffer_treat_as_8bit;
+	oldsample->treat_as_8bit = copybuffer_sampleinfo.treat_as_8bit;
+	oldsample->volume = copybuffer_sampleinfo.volume;
+	oldsample->finetune = copybuffer_sampleinfo.finetune;
+	oldsample->panning = copybuffer_sampleinfo.panning;
+	oldsample->relnote = copybuffer_sampleinfo.relnote;
 	sample_editor_unlock_sample();
 	ss = 0;
 	update_ie = 1;
@@ -1209,14 +1231,14 @@ sample_editor_reverse_clicked (void)
     sample_display_set_selection(sampledisplay, ss, se);
 }
 
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
 
 static void
 sample_editor_load_wav_main (int mode)
 { 
     /* Initialized global variables:
 
-       wavload_with_audiofile (TRUE or FALSE)
+       wavload_through_library (TRUE or FALSE)
        wavload_samplename     (the name the sample is going to get in the XM)
        wavload_frameCount     (length of the file /stereo /16bits)
 
@@ -1230,7 +1252,11 @@ sample_editor_load_wav_main (int mode)
     */
 
     void *sbuf, *sbuf_loadto;
+#if USE_SNDFILE
+    sf_count_t len;
+#else
     int len;
+#endif
     int i, count;
     float rate;
 
@@ -1238,7 +1264,7 @@ sample_editor_load_wav_main (int mode)
     
     len = 2 * wavload_frameCount * wavload_channelCount;
     if(!(sbuf = malloc(len))) {
-	error_error("Out of memory for sample data.");
+	error_error(_("Out of memory for sample data."));
 	goto errnobuf;
     }
 
@@ -1248,8 +1274,12 @@ sample_editor_load_wav_main (int mode)
 	sbuf_loadto = sbuf + len / 2;
     }
 
-    if(wavload_with_audiofile) {
+    if(wavload_through_library) {
+#if USE_SNDFILE
+	if(wavload_frameCount != sf_readf_short(wavload_file, sbuf_loadto, wavload_frameCount)) {
+#else
 	if(wavload_frameCount != afReadFrames(wavload_file, AF_DEFAULT_TRACK, sbuf_loadto, wavload_frameCount)) {
+#endif
 	    error_error(_("Read error."));
 	    goto errnodata;
 	}
@@ -1275,18 +1305,23 @@ sample_editor_load_wav_main (int mode)
     current_sample->sample.length = wavload_frameCount;
 
     // Initialize relnote and finetune such that sample is played in original speed
-    if(wavload_with_audiofile) {
+    if(wavload_through_library) {
+#if USE_SNDFILE
+	current_sample->treat_as_8bit = ((wavinfo.format & (SF_FORMAT_PCM_S8 | SF_FORMAT_PCM_U8)) != 0);
+	rate = wavinfo.samplerate;
+#else
 	rate = afGetRate(wavload_file, AF_DEFAULT_TRACK);
+#endif
     } else {
-	rate = 44100.0;
+	rate = wavload_rate;
     }
     xm_freq_note_to_relnote_finetune(rate,
-				     6 * 12 + 1, // at C-6
+				     4 * 12 + 1, // at C-4
 				     &current_sample->relnote,
 				     &current_sample->finetune);
 
     if(wavload_sampleWidth == 8) {
-	if(wavload_with_audiofile || wavload_unsignedwords) {
+	if(wavload_through_library || wavload_unsignedwords) {
 	    st_sample_8bit_signed_unsigned(sbuf_loadto, len / 2);
 	}
 	st_convert_sample(sbuf_loadto,
@@ -1295,7 +1330,7 @@ sample_editor_load_wav_main (int mode)
 			  16,
 			  len / 2);
     } else {
-	if(wavload_with_audiofile) {
+	if(wavload_through_library) {
 	    // I think that is what the virtualByteOrder stuff is for.
 	    // le_16_array_to_host_order(sbuf, len / 2);
 	} else {
@@ -1349,8 +1384,13 @@ sample_editor_load_wav_main (int mode)
     statusbar_update(STATUS_IDLE, FALSE);
     free(sbuf);
   errnobuf:
-    if(wavload_with_audiofile) {
+    if(wavload_through_library) {
+#if USE_SNDFILE
+	sf_close (wavload_file);
+#else
 	afCloseFile(wavload_file);
+#endif
+	wavload_file = NULL;
     }
 }
 
@@ -1468,6 +1508,7 @@ sample_editor_raw_sample_dialog_ok (GtkWidget *widget)
     wavload_endianness = find_current_toggle(wavload_raw_endian_w, 2);
     wavload_channelCount = find_current_toggle(wavload_raw_channels_w, 2) == 1 ? 2 : 1;
     wavload_unsignedwords = find_current_toggle(wavload_raw_signed_w, 2);
+    wavload_rate = atoi (gtk_entry_get_text(GTK_ENTRY(wavload_raw_rate)));
 
     gtk_widget_destroy(wavload_dialog);
 
@@ -1493,10 +1534,13 @@ sample_editor_open_raw_sample_dialog (const gchar *filename)
     GtkWidget *separator;
     GtkWidget *label;
     GtkWidget *thing;
-    static const char *resolutionlabels[] = { "8 bits", "16 bits", NULL };
-    static const char *signedlabels[] = { "Signed", "Unsigned", NULL };
-    static const char *endianlabels[] = { "Little-Endian", "Big-Endian", NULL };
-    static const char *channelslabels[] = { "Mono", "Stereo", NULL };
+    GtkWidget *combo;
+    GList *combo_items = NULL;
+    
+    static const char *resolutionlabels[] = { N_("8 bits"), N_("16 bits"), NULL };
+    static const char *signedlabels[] = { N_("Signed"), N_("Unsigned"), NULL };
+    static const char *endianlabels[] = { N_("Little-Endian"), N_("Big-Endian"), NULL };
+    static const char *channelslabels[] = { N_("Mono"), N_("Stereo"), NULL };
 
     wavload_filename = filename; // store for later usage
 
@@ -1568,6 +1612,40 @@ sample_editor_open_raw_sample_dialog (const gchar *filename)
     make_radio_group_full(channelslabels, box2, wavload_raw_channels_w,
 			  FALSE, TRUE, NULL, NULL);
 
+    // Standard sampling rates
+
+    combo_items = g_list_append (combo_items, "8000");
+    combo_items = g_list_append (combo_items, "8363");
+    combo_items = g_list_append (combo_items, "11025");
+    combo_items = g_list_append (combo_items, "12000");
+    combo_items = g_list_append (combo_items, "16000");
+    combo_items = g_list_append (combo_items, "16726");
+    combo_items = g_list_append (combo_items, "22050");
+    combo_items = g_list_append (combo_items, "24000");
+    combo_items = g_list_append (combo_items, "32000");
+    combo_items = g_list_append (combo_items, "33452");
+    combo_items = g_list_append (combo_items, "44100");
+    combo_items = g_list_append (combo_items, "48000");
+    
+    // Rate selection combo
+
+    box2 = gtk_hbox_new(FALSE, 4);
+    gtk_widget_show(box2);
+    gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, TRUE, 0);
+
+    thing = gtk_label_new(_("Sampling Rate:"));
+    gtk_widget_show(thing);
+    gtk_box_pack_start(GTK_BOX(box2), thing, FALSE, TRUE, 0);
+    add_empty_hbox(box2);
+
+    combo = gtk_combo_new ();
+    gtk_combo_set_popdown_strings (GTK_COMBO (combo), combo_items);
+    g_list_free (combo_items);  
+    gtk_box_pack_start(GTK_BOX(box2), combo, FALSE, TRUE, 0);
+    gtk_widget_show (combo);    
+    wavload_raw_rate = GTK_COMBO (combo)->entry; 
+    gtk_entry_set_text (GTK_ENTRY (wavload_raw_rate), _("8363")); // default
+    
     // The bottom of the box
 
     separator = gtk_hseparator_new ();
@@ -1596,10 +1674,18 @@ static void
 sample_editor_load_wav (void)
 {
     const gchar *fn = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_SAMPLE]));
+#if USE_SNDFILE != 1
     int sampleFormat;
+#endif
 
     gtk_widget_hide(fileops_dialogs[DIALOG_LOAD_SAMPLE]);
 
+    if(!file_selection_is_valid(fn)) {
+	/* No file was actually selected. */
+	gnome_error_dialog(_("No file selected."));
+	return;
+    }
+
     g_return_if_fail(current_sample != NULL);
 
     file_selection_save_path(fn, gui_settings.loadsmpl_path);
@@ -1610,14 +1696,19 @@ sample_editor_load_wav (void)
     else
 	wavload_samplename++;
 
+#if USE_SNDFILE
+    wavinfo.format = 0;
+    wavload_file = sf_open(fn, SFM_READ, &wavinfo);
+#else
     wavload_file = afOpenFile(fn, "r", NULL);
+#endif
     if(!wavload_file) {
 	FILE *f = fopen(fn, "r");
 	if(f) {
 	    fseek(f, 0, SEEK_END);
 	    wavload_frameCount = ftell(f);
 	    fclose(f);
-	    wavload_with_audiofile = FALSE;
+	    wavload_through_library = FALSE;
 	    sample_editor_open_raw_sample_dialog(fn);
 	} else {
 	    error_error(_("Can't read sample"));
@@ -1625,13 +1716,24 @@ sample_editor_load_wav (void)
 	return;
     }
 
-    wavload_with_audiofile = TRUE;
+    wavload_through_library = TRUE;
 
+#if USE_SNDFILE
+    wavload_frameCount = wavinfo.frames;
+#else
     wavload_frameCount = afGetFrameCount(wavload_file, AF_DEFAULT_TRACK);
+#endif
     if(wavload_frameCount > mixer->max_sample_length) {
 	error_warning(_("Sample is too long for current mixer module. Loading anyway."));
     }
 
+#if USE_SNDFILE
+
+    wavload_channelCount = wavinfo.channels;
+    wavload_sampleWidth = 16;
+    
+#else
+
     wavload_channelCount = afGetChannels(wavload_file, AF_DEFAULT_TRACK);
     afGetSampleFormat(wavload_file, AF_DEFAULT_TRACK, &sampleFormat, &wavload_sampleWidth);
 
@@ -1642,6 +1744,9 @@ sample_editor_load_wav (void)
     afSetVirtualByteOrder(wavload_file, AF_DEFAULT_TRACK, AF_BYTEORDER_LITTLEENDIAN);
 #endif
 
+#endif
+
+
     if((wavload_sampleWidth != 16 && wavload_sampleWidth != 8) || wavload_channelCount > 2) {
 	error_error(_("Can only handle 8 and 16 bit samples with up to 2 channels"));
 	goto errwrongformat;
@@ -1656,7 +1761,12 @@ sample_editor_load_wav (void)
     return;
 
   errwrongformat:
+#if USE_SNDFILE
+    sf_close(wavload_file);
+#else
     afCloseFile(wavload_file);
+#endif
+    wavload_file = NULL;
     return;
 }
 
@@ -1666,15 +1776,30 @@ sample_editor_save_wav_main (const gchar *fn,
 			     guint32 offset,
 			     guint32 length)
 {
+#if USE_SNDFILE
+    SNDFILE *outfile;
+    SF_INFO sfinfo;
+    int rate = 44100;
+#else
     AFfilehandle outfile;
     AFfilesetup outfilesetup;
     double rate = 44100.0;
-
+#endif
 
     statusbar_update(STATUS_SAVING_SAMPLE, TRUE);
 
     file_selection_save_path(fn, gui_settings.savesmpl_path);
 
+#if USE_SNDFILE
+    if(current_sample->treat_as_8bit) {
+	sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_U8;
+    } else {
+	sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
+    }
+    sfinfo.channels = 1;
+    sfinfo.samplerate = rate;
+    outfile = sf_open (fn, SFM_WRITE, &sfinfo);
+#else
     outfilesetup = afNewFileSetup();
     afInitFileFormat(outfilesetup, AF_FILE_WAVE);
     afInitChannels(outfilesetup, AF_DEFAULT_TRACK, 1);
@@ -1689,16 +1814,22 @@ sample_editor_save_wav_main (const gchar *fn,
     } else {
 	afInitSampleFormat(outfilesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
     }
-
     afInitRate(outfilesetup, AF_DEFAULT_TRACK, rate);
     outfile = afOpenFile(fn, "w", outfilesetup);
     afFreeFileSetup(outfilesetup);
+#endif
 
     if(!outfile) {
 	error_error(_("Can't open file for writing."));
 	return;
     }
 
+#if USE_SNDFILE
+    sf_writef_short (outfile, 
+		     current_sample->sample.data + offset,
+		     length);
+    sf_close(outfile);
+#else
     if(current_sample->treat_as_8bit) {
 	void *buf = malloc(1 * length);
 	g_assert(buf);
@@ -1709,6 +1840,7 @@ sample_editor_save_wav_main (const gchar *fn,
 			  length);
 	st_sample_8bit_signed_unsigned(buf,
 				       length);
+
 	afWriteFrames(outfile, AF_DEFAULT_TRACK,
 		      buf,
 		      length);
@@ -1718,8 +1850,8 @@ sample_editor_save_wav_main (const gchar *fn,
 		      current_sample->sample.data + offset,
 		      length);
     }
-
     afCloseFile(outfile);
+#endif
 
     statusbar_update(STATUS_SAMPLE_SAVED, FALSE);
 }
@@ -1744,6 +1876,12 @@ sample_editor_save_wav (void)
 
     gtk_widget_hide(fileops_dialogs[DIALOG_SAVE_SAMPLE]);
 
+    if(!file_selection_is_valid(fn)) {
+	/* No file was actually selected. */
+	gnome_error_dialog(_("No file selected."));
+	return;
+    }
+
     g_return_if_fail(current_sample != NULL);
     if(current_sample->sample.data == NULL)
 	return;
@@ -1768,7 +1906,7 @@ sample_editor_save_region_callback (gint reply,
 
     if(reply == 0) {
         // save region
-		sample_editor_save_wav_main((gchar*)data, rss, rse - rss);
+	sample_editor_save_wav_main((gchar*)data, rss, rse - rss);
     }
 }
 
@@ -1782,6 +1920,12 @@ sample_editor_save_region_wav (void)
 
     gtk_widget_hide(fileops_dialogs[DIALOG_SAVE_RGN_SAMPLE]);
 
+    if(!file_selection_is_valid(fn)) {
+	/* No file was actually selected. */
+	gnome_error_dialog(_("No file selected."));
+	return;
+    }
+
     g_return_if_fail(current_sample != NULL);
     if(current_sample->sample.data == NULL)
 	return;
@@ -1803,7 +1947,7 @@ sample_editor_save_region_wav (void)
     }
 }
 
-#endif /* NO_AUDIOFILE */
+#endif /* USE_SNDFILE || !defined (NO_AUDIOFILE) */
 
 /* ============================ Sampling functions coming up -------- */
 
@@ -1923,7 +2067,7 @@ sample_editor_sampled (void *dest,
 	if(currentoffs == recordbuflen) {
 	    struct recordbuf *newbuf = malloc(sizeof(struct recordbuf) + recordbuflen * 2);
 	    if(!newbuf) {
-		error_error("Out of memory while sampling!");
+		error_error(_("Out of memory while sampling!"));
 		sampling = 0;
 		break;
 	    }
@@ -2014,7 +2158,7 @@ sample_editor_ok_clicked (void)
     current_sample->treat_as_8bit = FALSE;
 
     xm_freq_note_to_relnote_finetune(rate,
-				     6 * 12 + 1, // at C-6
+				     4 * 12 + 1, // at C-4
 				     &current_sample->relnote,
 				     &current_sample->finetune);
 
@@ -2038,7 +2182,7 @@ static void
 sample_editor_lrvol (GtkWidget *widget,
                      gpointer data)
 {
-    int mode = (int)data;
+    int mode = GPOINTER_TO_INT(data);
 
     switch(mode)
     {
@@ -2056,7 +2200,7 @@ static void
 sample_editor_open_volume_ramp_dialog (void)
 {
     GtkWidget *mainbox, *box1, *thing;
-    int i;
+    gint i;
     const char *labels1[] = {
 	_("Normalize"),
 	_("Execute"),
@@ -2146,7 +2290,8 @@ sample_editor_open_volume_ramp_dialog (void)
 	gtk_widget_show(thing);
 	gtk_box_pack_start(GTK_BOX(box1), thing, TRUE, TRUE, 0);
 	gtk_signal_connect(GTK_OBJECT(thing), "clicked",
-			   GTK_SIGNAL_FUNC(sample_editor_perform_ramp), (gpointer)i);
+			   GTK_SIGNAL_FUNC(sample_editor_perform_ramp),
+			   GINT_TO_POINTER(i));
     }
 
     gtk_widget_show (volrampwindow);
@@ -2166,7 +2311,7 @@ static void
 sample_editor_perform_ramp (GtkWidget *w,
 			    gpointer data)
 {
-    int action = (int)data;
+    int action = GPOINTER_TO_INT(data);
     double left, right;
     const int ss = sampledisplay->sel_start, se = sampledisplay->sel_end;
     int i;
