@@ -26,6 +26,10 @@
 
 #include "sample-display.h"
 
+#ifndef NO_GDK_PIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
+
 #define SCOPE_GROUP(obj)          GTK_CHECK_CAST (obj, scope_group_get_type (), ScopeGroup)
 #define SCOPE_GROUP_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, scope_group_get_type (), ScopeGroupClass)
 #define IS_SCOPE_GROUP(obj)       GTK_CHECK_TYPE (obj, scope_group_get_type ())
@@ -40,10 +44,14 @@ struct _ScopeGroup
     GtkWidget *table;
     SampleDisplay *scopes[32];
     GtkWidget *scopebuttons[32];
+#ifndef NO_GDK_PIXBUF
+    GtkWidget *mutedpic[32];
+#endif
     int numchan;
     int scopes_on;
     int update_freq;
     int gtktimer;
+    gint32 on_mask;
 };
 
 struct _ScopeGroupClass
@@ -54,8 +62,13 @@ struct _ScopeGroupClass
 guint
 scope_group_get_type (void);
 
+#ifndef NO_GDK_PIXBUF
+GtkWidget *
+scope_group_new (GdkPixbuf *mutedpic);
+#else
 GtkWidget *
 scope_group_new (void);
+#endif
 
 void
 scope_group_set_num_channels (ScopeGroup *s, int num_channels);
