@@ -318,21 +318,35 @@ st_num_save_patterns (XM *xm)
 {
     int i, n;
 
-    if(gui_settings.save_all_patterns) {
-	for(i = 0, n = 0; i < 256; i++) {
-	    if(!st_is_empty_pattern(&xm->patterns[i]))
-		n = i;
-	}
-    } else {
-	for(i = 0, n = 0; i < 256; i++) {
-	    if(xm->pattern_order_table[i] > n)
-		n = xm->pattern_order_table[i];
-	}
+    for(i = 0, n = 0; i < 256; i++) {
+	if(!st_is_empty_pattern(&xm->patterns[i]))
+	    n = i;
     }
 
     return n + 1;
 }
 
+void
+st_copy_instrument (STInstrument *src, STInstrument *dest)
+{
+    int i;
+    guint32 length;
+    GMutex *lock[16];
+
+    st_clean_instrument(dest, NULL);
+
+    for(i = 0; i < sizeof(dest->samples) / sizeof(dest->samples[0]); i++)
+	lock[i] = dest->samples[i].sample.lock; // Preserve pointers to GMutex'es from modification
+    memcpy(dest, src, sizeof(STInstrument));
+    for(i = 0; i < sizeof(src->samples) / sizeof(src->samples[0]); i++){
+	if ((length = dest->samples[i].sample.length * sizeof(dest->samples[i].sample.data[0]))){
+	    dest->samples[i].sample.data = malloc(length);
+	    memcpy(dest->samples[i].sample.data, src->samples[i].sample.data, length);
+	}
+	dest->samples[i].sample.lock = lock[i];
+    }
+}
+
 void
 st_clean_instrument (STInstrument *instr,
 		     const char *name)
@@ -445,9 +459,11 @@ st_find_first_unused_and_empty_pattern (XM *xm)
 {
     int i;
 
-    for(i = 0; i < sizeof(xm->patterns) / sizeof(xm->patterns[0]); i++)
-	if(!st_is_pattern_used_in_song(xm, i) && st_is_empty_pattern(&xm->patterns[i]))
+    for(i = 0; i < sizeof(xm->patterns) / sizeof(xm->patterns[0]); i++) {
+	if(!st_is_pattern_used_in_song(xm, i) && st_is_empty_pattern(&xm->patterns[i])) {
 	    return i;
+	}
+    }
 
     return -1;
 }
@@ -457,10 +473,13 @@ st_is_empty_pattern (XMPattern *p)
 {
     int i;
     
-    for(i = 0; i < 32; i++)
-	if(p->channels[i])
-	    if(!st_is_empty_track(p->channels[i], p->length))
+    for(i = 0; i < 32; i++) {
+	if(p->channels[i]) {
+	    if(!st_is_empty_track(p->channels[i], p->length)) {
 		return 0;
+	    }
+	}
+    }
 
     return 1;
 }
@@ -469,10 +488,12 @@ gboolean
 st_is_empty_track (XMNote *notes,
 		   int length)
 {
-    for(; length >= 0; length--)
+    for(; length > 0; length--, notes++) {
 	if(notes->note != 0 || notes->instrument != 0 || notes->volume != 0
-	   || notes->fxtype != 0 || notes->fxparam != 0)
+	   || notes->fxtype != 0 || notes->fxparam != 0) {
 	    return 0;
+	}
+    }
 
     return 1;
 }
@@ -510,6 +531,61 @@ st_exchange_patterns (XM *xm,
     }
 }
 
+gboolean
+st_check_if_odd_are_not_empty (XMPattern *p)
+{
+    int i, j;
+
+    for(i = 0; i < 32 && p->channels[i]; i++)
+	for(j = 1; j < p->length; j += 2)
+	    if((p->channels[i][j].note && p->channels[i][j].instrument) ||
+	       (p->channels[i][j].volume > 15) ||
+	        p->channels[i][j].fxtype || p->channels[i][j].fxparam)
+		return TRUE;
+
+    return FALSE;
+}
+
+void
+st_shrink_pattern (XMPattern *p)
+{
+    int i, j, length = p->length;
+    
+    for(i = 0; i < 32 && p->channels[i]; i++){
+	for(j = 1; j <= (length - 1) / 2; j++)
+	    memcpy(&p->channels[i][j], &p->channels[i][2 * j], sizeof(XMNote));
+	for(;j < p->alloc_length; j++){/* clear the rest of the pattern */
+	    p->channels[i][j].note = 0;
+	    p->channels[i][j].instrument = 0;
+	    p->channels[i][j].volume = 0;
+	    p->channels[i][j].fxtype = 0;
+	    p->channels[i][j].fxparam = 0;
+	}
+    }
+    
+    st_set_pattern_length(p, (length - 1) / 2 + 1);
+}
+
+void
+st_expand_pattern (XMPattern *p)
+{
+    int i, j, length = MIN(p->length * 2, 256);
+
+    st_set_pattern_length(p, length);
+
+    for(i = 0; i < 32 && p->channels[i]; i++){
+	for(j = length / 2 - 1; j >= 0; j--){
+	    /* copy to even positions and clear odd */
+	    memcpy(&p->channels[i][2 * j], &p->channels[i][j], sizeof(XMNote));
+	    p->channels[i][2 * j + 1].note = 0;
+	    p->channels[i][2 * j + 1].instrument = 0;
+	    p->channels[i][2 * j + 1].volume = 0;
+	    p->channels[i][2 * j + 1].fxtype = 0;
+	    p->channels[i][2 * j + 1].fxparam = 0;
+	}
+    }
+}
+
 void
 st_convert_sample (void *src,
 		   void *dst,
@@ -566,7 +642,7 @@ st_sample_16bit_signed_unsigned (gint16 *data,
 				 int count)
 {
     while(count) {
-	*data = *data++ + 65536;
+	*data = *data++ + 32768;
 	count--;
     }
 }
