@@ -38,10 +38,12 @@
 #include "preferences.h"
 #include "tracker-settings.h"
 #include "menubar.h"
+#include "scope-group.h"
 
 Tracker *tracker;
 GtkWidget *trackersettings;
 GtkWidget *vscrollbar;
+ScopeGroup *scopegroup;
 
 static GtkWidget *hscrollbar;
 
@@ -192,7 +194,7 @@ show_editmode_status(void)
         break;
     default:
         if(note->volume>0xf) {
-            sprintf(tmp_buf,"%s", vol_fx_commands[((note->volume&0xf0)-0x60)>>4]);
+            sprintf(tmp_buf,"%s", _(vol_fx_commands[((note->volume&0xf0)-0x60)>>4]));
             strcat(track_editor_editmode_status_ed_buf, tmp_buf);
         } else
             strcat(track_editor_editmode_status_ed_buf, _("None"));
@@ -216,18 +218,18 @@ show_editmode_status(void)
     {
     case 0:
         if(note->fxparam)
-            sprintf(tmp_buf, "%s", fx_commands[note->fxtype]);
+            sprintf(tmp_buf, "%s", _(fx_commands[note->fxtype]));
         else
-            sprintf(tmp_buf, "None ]");
+            sprintf(tmp_buf, _("None ]"));
         break;
     case 14:
         switch((note->fxparam&0xf0)>>4)
         {
         case 0: case 8: case 15:
-            sprintf(tmp_buf, "None ]");
+            sprintf(tmp_buf, _("None ]"));
             break;
         default:
-            sprintf(tmp_buf, "%s", e_fx_commands[(note->fxparam&0xf0)>>4]);
+            sprintf(tmp_buf, "%s", _(e_fx_commands[(note->fxparam&0xf0)>>4]));
             break;
         }
         break;
@@ -241,17 +243,17 @@ show_editmode_status(void)
             sprintf(tmp_buf, "Extra fine porta down");
             break;
         default:
-            sprintf(tmp_buf, "None ]");
+            sprintf(tmp_buf, _("None ]"));
             break;
         }
         break;
     case 18: case 19: case 22: case 23: case 24:
     case 28: case 30: case 31: case 32: case 34:
-        sprintf(tmp_buf, "None ]");
+        sprintf(tmp_buf, _("None ]"));
         break;
 
     default:
-        sprintf(tmp_buf, "%s", fx_commands[note->fxtype]);
+        sprintf(tmp_buf, "%s", _(fx_commands[note->fxtype]));
         break;
     }
     strcat(track_editor_editmode_status_ed_buf, tmp_buf);
@@ -502,6 +504,8 @@ track_editor_do_the_note_key (int notekeymeaning,
 			      guint32 xkeysym,
 			      int modifiers)
 {
+    static int playchan = 0;
+
     int j, n;
     int note;
 
@@ -512,11 +516,20 @@ track_editor_do_the_note_key (int notekeymeaning,
 
     if(pressed) {
 	if(note_running[note] == -1) {
-	    gui_play_note(tracker->cursor_ch, note);
-	    note_running[note] = tracker->cursor_ch;
 	    if(jazz_enabled) {
-		n = track_editor_find_next_jazz_channel(tracker->cursor_ch, note);
-		tracker_step_cursor_channel(tracker, n - tracker->cursor_ch);
+		if(GTK_TOGGLE_BUTTON(editing_toggle)->active) {
+		    gui_play_note(tracker->cursor_ch, note);
+		    note_running[note] = tracker->cursor_ch;
+		    n = track_editor_find_next_jazz_channel(tracker->cursor_ch, note);
+		    tracker_step_cursor_channel(tracker, n - tracker->cursor_ch);
+		} else {
+		    playchan = track_editor_find_next_jazz_channel(playchan, note);
+		    gui_play_note(playchan, note);
+		    note_running[note] = playchan;
+		}
+	    } else {
+		gui_play_note(tracker->cursor_ch, note);
+		note_running[note] = tracker->cursor_ch;
 	    }
 	}
     } else {
@@ -545,14 +558,18 @@ track_editor_handle_keys (int shift,
 			  guint32 keyval,
 			  gboolean pressed)
 {
-    int i, m;
+    static int current_channel = -1;
+
+    int i, m, tip;
     Tracker *t = tracker;
     gboolean handled = FALSE;
 
-    if(t->cursor_item == 0) {
-	m = i = keys_get_key_meaning(keyval, ENCODE_MODIFIERS(shift, ctrl, alt));
+    m = i = keys_get_key_meaning(keyval, ENCODE_MODIFIERS(shift, ctrl, alt));
+    tip = KEYS_MEANING_TYPE(i);
+
+    if(t->cursor_item == 0 &&  tip <= KEYS_MEANING_KEYOFF) {
 	if(i != -1) {
-	    switch(KEYS_MEANING_TYPE(i)) {
+	    switch(tip) {
 	    case KEYS_MEANING_NOTE:
 		i += 12 * gui_get_current_octave_value() + 1;
 		if(i < 96) {
@@ -582,8 +599,24 @@ track_editor_handle_keys (int shift,
 	}
     }
 
-    if(!pressed)
+    if(!pressed){
+	if (tip == KEYS_MEANING_CH){
+	    current_channel = -1;
+	    return TRUE;
+	}
 	return FALSE;
+    }
+
+    switch (tip){
+    case KEYS_MEANING_FUNC:
+	m = gui_get_current_jump_value ();
+	if ((m += KEYS_MEANING_VALUE(i) ? -1 : 1)  <= 16 &&
+	     m >= 0) gui_set_jump_value (m);
+	return TRUE;
+    case KEYS_MEANING_CH:
+	tracker_set_cursor_channel (t, current_channel = KEYS_MEANING_VALUE(i));
+	return TRUE;
+    }
 
     switch (keyval) {
     case GDK_Up:
@@ -752,8 +785,25 @@ track_editor_handle_keys (int shift,
         }
         break;
 
+    case GDK_Escape:
+       if (shift){
+           tracker_set_cursor_item (t, 0);
+           handled = TRUE;
+       }
+       break;
+
+    case ' ':
+	if (current_channel >= 0){
+	    if (current_channel > (t->num_channels - 1))
+		current_channel = t->num_channels - 1;
+	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(scopegroup->scopebuttons[current_channel]),
+					 !GTK_TOGGLE_BUTTON(scopegroup->scopebuttons[current_channel])->active);
+	    handled = TRUE;
+	}
+	break;
+
     default:
-	if(!ctrl) {
+	if(!ctrl && !alt) {
 	    if(GTK_TOGGLE_BUTTON(editing_toggle)->active) {
 		handled = track_editor_handle_column_input(t, keyval);
 	    }
@@ -1228,7 +1278,12 @@ tracker_timeout (gpointer data)
     double display_songtime;
     audio_player_pos *p;
 
-    g_assert(current_driver_object);
+    if(current_driver_object == NULL) {
+	/* Can happen when audio thread stops on its own. Note that
+	 * tracker_stop_updating() is called in
+	 * gui.c::read_mixer_pipe(). */
+	return TRUE;
+    }
 
     display_songtime = current_driver->get_play_time(current_driver_object);
 
