@@ -28,9 +28,10 @@
 
 #include "tracker.h"
 #include "main.h"
+#include "gui.h"
 #include "gui-settings.h"
 
-const char * const notenames[96] = {
+const char * const notenames[4][96] = {{
     "C-0", "C#0", "D-0", "D#0", "E-0", "F-0", "F#0", "G-0", "G#0", "A-0", "A#0", "H-0",
     "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "H-1",
     "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "H-2",
@@ -39,7 +40,34 @@ const char * const notenames[96] = {
     "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "H-5",
     "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "H-6",
     "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "H-7",
-};
+},{
+    "C-0", "Db0", "D-0", "Eb0", "E-0", "F-0", "Gb0", "G-0", "Ab0", "A-0", "B-0", "H-0",
+    "C-1", "Db1", "D-1", "Eb1", "E-1", "F-1", "Gb1", "G-1", "Ab1", "A-1", "B-1", "H-1",
+    "C-2", "Db2", "D-2", "Eb2", "E-2", "F-2", "Gb2", "G-2", "Ab2", "A-2", "B-2", "H-2",
+    "C-3", "Db3", "D-3", "Eb3", "E-3", "F-3", "Gb3", "G-3", "Ab3", "A-3", "B-3", "H-3",
+    "C-4", "Db4", "D-4", "Eb4", "E-4", "F-4", "Gb4", "G-4", "Ab4", "A-4", "B-4", "H-4",
+    "C-5", "Db5", "D-5", "Eb5", "E-5", "F-5", "Gb5", "G-5", "Ab5", "A-5", "B-5", "H-5",
+    "C-6", "Db6", "D-6", "Eb6", "E-6", "F-6", "Gb6", "G-6", "Ab6", "A-6", "B-6", "H-6",
+    "C-7", "Db7", "D-7", "Eb7", "E-7", "F-7", "Gb7", "G-7", "Ab7", "A-7", "B-7", "H-7",
+},{
+    "C-0", "C#0", "D-0", "D#0", "E-0", "F-0", "F#0", "G-0", "G#0", "A-0", "A#0", "B-0",
+    "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
+    "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2",
+    "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3", "G-3", "G#3", "A-3", "A#3", "B-3",
+    "C-4", "C#4", "D-4", "D#4", "E-4", "F-4", "F#4", "G-4", "G#4", "A-4", "A#4", "B-4",
+    "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "B-5",
+    "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "B-6",
+    "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "B-7",
+},{
+    "C-0", "Db0", "D-0", "Eb0", "E-0", "F-0", "Gb0", "G-0", "Ab0", "A-0", "Bb0", "B-0",
+    "C-1", "Db1", "D-1", "Eb1", "E-1", "F-1", "Gb1", "G-1", "Ab1", "A-1", "Bb1", "B-1",
+    "C-2", "Db2", "D-2", "Eb2", "E-2", "F-2", "Gb2", "G-2", "Ab2", "A-2", "Bb2", "B-2",
+    "C-3", "Db3", "D-3", "Eb3", "E-3", "F-3", "Gb3", "G-3", "Ab3", "A-3", "Bb3", "B-3",
+    "C-4", "Db4", "D-4", "Eb4", "E-4", "F-4", "Gb4", "G-4", "Ab4", "A-4", "Bb4", "B-4",
+    "C-5", "Db5", "D-5", "Eb5", "E-5", "F-5", "Gb5", "G-5", "Ab5", "A-5", "Bb5", "B-5",
+    "C-6", "Db6", "D-6", "Eb6", "E-6", "F-6", "Gb6", "G-6", "Ab6", "A-6", "Bb6", "B-6",
+    "C-7", "Db7", "D-7", "Eb7", "E-7", "F-7", "Gb7", "G-7", "Ab7", "A-7", "Bb7", "B-7",
+}};
 
 static void init_display(Tracker *t, int width, int height);
 
@@ -52,7 +80,8 @@ static const int default_colors[] = {
     230, 230, 230,
     170, 170, 200,
     230, 200, 0,
-    250, 100, 50
+    250, 100, 50,
+    250, 200, 50,
 };
 
 enum {
@@ -231,6 +260,18 @@ tracker_step_cursor_item (Tracker *t,
     }
 }
 
+void
+tracker_set_cursor_item (Tracker *t,
+                         int item)
+{
+    if (item < 0) item = 0;
+    if (item > 7) item = 7;
+
+    t->cursor_item = item;
+    adjust_xpanning(t);
+    gtk_widget_queue_draw(GTK_WIDGET(t));
+}
+ 
 void
 tracker_step_cursor_channel (Tracker *t,
 			     int direction)
@@ -244,6 +285,27 @@ tracker_step_cursor_channel (Tracker *t,
 
     adjust_xpanning(t);
 
+    if(t->inSelMode) {
+       /* Force re-draw of patterns in block selection mode */
+       gtk_widget_queue_draw(GTK_WIDGET(t));
+    } else {
+       tracker_idle_draw(t);
+    }
+}
+
+void
+tracker_set_cursor_channel (Tracker *t,
+                            int channel)
+{
+    t->cursor_ch = channel;
+
+    if(t->cursor_ch < 0)
+       t->cursor_ch = 0;
+    else if(t->cursor_ch >= t->num_channels)
+       t->cursor_ch = t->num_channels - 1;
+
+    adjust_xpanning(t);
+
     if(t->inSelMode) {
 	/* Force re-draw of patterns in block selection mode */
 	gtk_widget_queue_draw(GTK_WIDGET(t));
@@ -363,9 +425,10 @@ note2string (XMNote *note,
     const char *hexmap = gui_settings.tracker_upcase ? hexmapU : hexmapL;
 
     if(note->note > 0 && note->note < 97) {
-        buf[0] = notenames[note->note-1][0];
-        buf[1] = notenames[note->note-1][1];
-        buf[2] = notenames[note->note-1][2];
+	int index = (gui_settings.sharp ? 0 : 1) + (gui_settings.bh ? 2 : 0);
+        buf[0] = notenames[index][note->note-1][0];
+        buf[1] = notenames[index][note->note-1][1];
+        buf[2] = notenames[index][note->note-1][2];
     } else if(note->note == 97) {
         buf[0] = '[';
         buf[1] = '-';
@@ -525,7 +588,7 @@ print_notes_line (GtkWidget *widget,
     gdk_draw_string(win, t->font, t->notes_gc, 5, y, buf);
     
     /* The notes */
-    for(numch += ch, bufpt = buf; ch < numch; ch++, bufpt += 15) {
+    for(numch += ch, bufpt = buf; ch < numch; ch++, bufpt += 14) {
 	note2string(&t->curpattern->channels[ch][row], bufpt);
     }	
 
@@ -574,10 +637,10 @@ print_notes_and_bars (GtkWidget *widget,
 
     /* Draw the separation bars */
     gdk_gc_set_foreground(t->misc_gc, &t->colors[TRACKERCOL_BARS]);
-    x1 = t->disp_startx + t->disp_chanwidth - 2;
-    y1 = t->disp_starty + n1 * t->fonth;
-    h = (n2 - n1 + 1) * t->fonth;
-    for(i = 1; i <= t->disp_numchans; i++, x1 += t->disp_chanwidth) {
+    x1 = t->disp_startx - 2;
+    y1 = t->disp_starty + n1 * t->fonth - t->fonth;
+    h = (n2 - n1 + 2) * t->fonth;
+    for(i = 0; i <= t->disp_numchans; i++, x1 += t->disp_chanwidth) {
 	gdk_draw_line(win, t->misc_gc, x1, y1, x1, y1+h);
     }
 }
@@ -596,12 +659,11 @@ print_channel_numbers (GtkWidget *widget,
 
     gdk_gc_set_foreground(t->misc_gc, &t->colors[TRACKERCOL_CHANNUMS]);
 
-    x = t->disp_startx + t->disp_chanwidth - (2 * t->fontw);
-    y = t->disp_starty + t->font->ascent + t->baselineskip;
-
+    x = t->disp_startx + (t->disp_chanwidth - (2 * t->fontw)) / 2; 
+    y = t->disp_starty + t->font->ascent + t->baselineskip - t->fonth;
     for(i = 1; i <= t->disp_numchans; i++, x += t->disp_chanwidth) {
 	sprintf(buf, "%2d", i + t->leftchan);
-	gdk_draw_rectangle(win, t->bg_gc, TRUE, x, t->disp_starty, 2*t->fontw, t->fonth);
+	gdk_draw_rectangle(win, t->bg_gc, TRUE, x, t->disp_starty - t->fonth, 2*t->fontw, t->fonth);
 	gdk_draw_string(win, t->font, t->misc_gc, x, y, buf);
     }
 }
@@ -652,7 +714,12 @@ print_cursor (GtkWidget *widget,
 
     x = x * t->fontw + t->disp_startx + (t->cursor_ch - t->leftchan) * t->disp_chanwidth - 1; 
 
-    gdk_gc_set_foreground(t->misc_gc, &t->colors[TRACKERCOL_CURSOR]);
+
+	if (GTK_TOGGLE_BUTTON(editing_toggle)->active)
+	    gdk_gc_set_foreground(t->misc_gc, &t->colors[TRACKERCOL_CURSOR_EDIT]);
+	else
+		gdk_gc_set_foreground(t->misc_gc, &t->colors[TRACKERCOL_CURSOR]);
+
     gdk_draw_rectangle(win, t->misc_gc, FALSE, x, y, width * t->fontw, t->fonth - 1);
 }
 
@@ -783,7 +850,7 @@ tracker_size_request (GtkWidget *widget,
 		      GtkRequisition *requisition)
 {
     Tracker *t = TRACKER(widget);
-    requisition->width = 15 * t->fontw + 3*t->fontw + 10;
+    requisition->width = 14 * t->fontw + 3*t->fontw + 10;
     requisition->height = 11*t->fonth;
 }
 
@@ -804,9 +871,10 @@ init_display (Tracker *t,
     if(!(t->disp_rows % 2))
 	t->disp_rows--;
     t->disp_cursor = t->disp_rows / 2;
-    t->disp_starty = (height - t->fonth * t->disp_rows) / 2;
+    t->disp_starty = (height - t->fonth * t->disp_rows) / 2 + t->fonth;
+	t->disp_rows--;
 
-    t->disp_chanwidth = 15 * t->fontw;
+    t->disp_chanwidth = 14 * t->fontw;
     u = width - line_numbers_space - 10;
     t->disp_numchans = u / t->disp_chanwidth;
     
@@ -1074,7 +1142,8 @@ tracker_button_press (GtkWidget      *widget,
     t = TRACKER(widget);
     gdk_window_get_pointer (event->window, &x, &y, &state);
 
-    if(t->mouse_selecting && event->button != 1) {
+    if(t->mouse_selecting && event->button != 1 && event->button != 5
+       && event->button != 4) { /* Wheel buttons must work independently on selection */
 	t->mouse_selecting = 0;
     } else if(!t->mouse_selecting) {
 	t->button = event->button;		
@@ -1114,6 +1183,22 @@ tracker_button_press (GtkWidget      *widget,
 	    }
 	}
     }
+    if (event->button == 4)
+    {
+	if  (event->state & GDK_SHIFT_MASK)
+	{
+	    if (t->leftchan > 0) tracker_set_xpanning (t, t->leftchan - 1);
+	} else
+	    if (t->patpos > 0) tracker_step_cursor_row (t, -1);
+    }
+    if (event->button == 5)
+    {
+	if  (event->state & 0x1)
+	{
+	    if (t->leftchan + t->disp_numchans < t->num_channels) tracker_set_xpanning (t, t->leftchan + 1);
+	} else
+	    if (t->patpos < t->curpattern->length - 1) tracker_step_cursor_row (t, 1);
+    }
     return TRUE;
 }
 
