@@ -44,7 +44,8 @@ enum {
     TRACKERCOL_BARS,
     TRACKERCOL_CHANNUMS,
     TRACKERCOL_CURSOR,
-    TRACKERCOL_LAST
+    TRACKERCOL_CURSOR_EDIT,
+    TRACKERCOL_LAST,
 };
 
 struct _Tracker
@@ -93,7 +94,7 @@ struct _TrackerClass
     void (*mainmenu_blockmark_set)(Tracker *t, int state);
 };
 
-extern const char * const notenames[96];
+extern const char * const notenames[4][96];
 
 guint		tracker_get_type            (void);
 GtkWidget*     	tracker_new                 (void);
@@ -112,6 +113,8 @@ void           	tracker_redraw_current_row  (Tracker *t);
 /* These are the navigation functions. */
 void           	tracker_step_cursor_item    (Tracker *t, int direction);
 void           	tracker_step_cursor_channel (Tracker *t, int direction);
+void            tracker_set_cursor_channel  (Tracker *t, int channel);
+void            tracker_set_cursor_item     (Tracker *t, int item);
 void           	tracker_set_patpos          (Tracker *t, int row);
 void           	tracker_step_cursor_row     (Tracker *t, int direction);
 
