@@ -4,7 +4,7 @@
  *
  * Copyright (C) 1994-1998 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
  * Copyright (C) 1998 Tammo Hinrichs <opencp@gmx.net>
- * Copyright (C) 1998-1999 Michael Krause
+ * Copyright (C) 1998-2001 Michael Krause
  *
  * Based on the source code of "OpenCP", the DOS module player,
  * written by Niklas Beisert and now maintained by Tammo Hinrichs
@@ -38,6 +38,7 @@
 #include "audio.h"
 
 int player_songpos, player_patpos;
+int player_tempo, player_bpm;
 gboolean player_looped;
 static double current_time;
 static int xmplayer_playmode;
@@ -48,9 +49,6 @@ enum {
     PLAYING_NOTE,
 };
 
-#define PITCH_NOTE        (16 * 4 * 4)
-#define PITCH_OCTAVE      (12 * PITCH_NOTE) 
-
 static inline int
 env_length (STEnvelope *env)
 {
@@ -133,12 +131,13 @@ enum
   xmpCmdPanning=8,xmpCmdOffset=9,xmpCmdVolSlide=10,xmpCmdJump=11,
   xmpCmdVolume=12,xmpCmdBreak=13,xmpCmdSpeed=15,xmpCmdGVolume=16,
   xmpCmdGVolSlide=17,xmpCmdKeyOff=20,xmpCmdEnvPos=21,xmpCmdPanSlide=25,
-  xmpCmdMRetrigger=27,xmpCmdTremor=29,xmpCmdXPorta=33,xmpCmpSFilter=36,
+  xmpCmdSetFReso=26, xmpCmdMRetrigger=27,xmpCmdTremor=29,xmpCmdXPorta=33,
+  xmpCmdSetFCutoff=35, xmpCmdSFilter=36,
   xmpCmdFPortaU=37,xmpCmdFPortaD=38,xmpCmdGlissando=39,xmpCmdVibType=40,
   xmpCmdSFinetune=41,xmpCmdPatLoop=42,xmpCmdTremType=43,xmpCmdSPanning=44,
   xmpCmdRetrigger=45,xmpCmdFVolSlideU=46, xmpCmdFVolSlideD=47,
-  xmpCmdNoteCut=48,xmpCmdDelayNote=49,xmpCmdPatDelay=50, xmpCmdSync1=28,
-  xmpCmdSync2=32, xmpCmdSync3=51,
+  xmpCmdNoteCut=48,xmpCmdDelayNote=49,xmpCmdPatDelay=50,
+  xmpCmdSetFHFCutoff=31, xmpCmdSetFHFReso=32,
   xmpVCmdVol0x=1,xmpVCmdVol1x=2,xmpVCmdVol2x=3,xmpVCmdVol3x=4,xmpVCmdVol40=5,
   xmpVCmdVolSlideD=6,xmpVCmdVolSlideU=7,xmpVCmdFVolSlideD=8,
   xmpVCmdFVolSlideU=9,xmpVCmdVibRate=10,xmpVCmdVibDep=11,xmpVCmdPanning=12,
@@ -155,6 +154,8 @@ typedef struct channel
     gint32 chPitch;          /* Pitch really means 'period' in nonlinear and module frequencies */
     gint32 chFinalPitch;
     int curnote;
+    long chCutoff;
+    long chReso;
 
     guint8 chCurIns;
     guint8 chLastIns;
@@ -212,6 +213,7 @@ typedef struct channel
     int nextstop;
     STSample *nextsamp;
     int nextpos;
+    int sampleplayend; /* don't play all of the sample, but stop at (here) */
     STSample *cursamp;
     STInstrument *curins;
     int hacksample; /* if 1, then simply play the sample pointed to by cursamp */
@@ -222,10 +224,9 @@ static channel channels[32];
 static guint8 globalvol;
 
 static guint8 curtick;
-static guint8 curtempo;
 static guint8 tick0;
 
-static int currow;
+static int currow, play_only_row;
 static XMPattern *curpattern;
 static int patlen;
 static int curord;
@@ -248,7 +249,6 @@ static guint8 procins;
 static guint8 procvol;
 static guint8 proccmd;
 static guint8 procdat;
-static int curbpm;
 
 static int realgvol;
 
@@ -437,15 +437,15 @@ xm_player_start_note (channel *ch,
 }
 
 static gint32
-xm_player_get_note_frq (channel *ch)
+xm_player_get_note_pitch (channel *ch)
 {
     if(!ismod) {
-	gint32 frq = 48*256 - (((procnot - 1) << 8) - ch->chCurNormNote);
-	
+	gint32 pitch = 48*256 - (((procnot - 1) << 8) - ch->chCurNormNote);
+
 	if(linearfreq)
-	    return frq;
+	    return pitch;
 	else
-	    return mcpGetFreq6848(frq);
+	    return mcpGetFreq6848(pitch);
     } else {
 	int note = procnot - 1 - 36;
 
@@ -494,6 +494,9 @@ xm_player_playnote_protracker (channel *ch)
 	    gint32 nn;
 	    ch->nextstop=1;
 
+	    if(!ch->cursamp)
+		return;
+
 	    ch->nextsamp = ch->cursamp;
 
 	    /* CurNormNote is only relevant in FastTracker mode */
@@ -502,9 +505,10 @@ xm_player_playnote_protracker (channel *ch)
 		nn = -ch->cursamp->relnote*256 - (gint16)(procdat << 4) + 0x80;
 	    ch->chCurNormNote = nn;
 		
-	    ch->chPitch = ch->chFinalPitch = ch->chPortaToPitch = xm_player_get_note_frq(ch);
+	    ch->chPitch = ch->chFinalPitch = ch->chPortaToPitch = xm_player_get_note_pitch(ch);
 
 	    ch->nextpos=0;
+	    ch->sampleplayend=-1;
 
 	    if (proccmd==xmpCmdOffset) {
 		if (procdat!=0)
@@ -520,7 +524,7 @@ xm_player_playnote_protracker (channel *ch)
 	    ch->chMRetrigPos=0;
 	    ch->chTremorPos=0;
 	} else {
-	    ch->chPortaToPitch = xm_player_get_note_frq(ch);
+	    ch->chPortaToPitch = xm_player_get_note_pitch(ch);
 	}
     }
 
@@ -558,7 +562,6 @@ xm_player_playnote_fasttracker (channel *ch)
 
     if (procnot==97) {
 	procnot=0;
-	procins=0;
 	keyoff=1;
     }
 
@@ -592,6 +595,9 @@ xm_player_playnote_fasttracker (channel *ch)
 		    return;
 	    }
 
+	    if(!ch->cursamp)
+		return;
+
 	    ch->nextsamp = ch->cursamp;
 
 	    /* CurNormNote is only relevant in FastTracker mode */
@@ -600,9 +606,10 @@ xm_player_playnote_fasttracker (channel *ch)
 		nn = -ch->cursamp->relnote*256 - (gint16)(procdat << 4) + 0x80;
 	    ch->chCurNormNote = nn;
 		
-	    ch->chPitch = ch->chFinalPitch = ch->chPortaToPitch = xm_player_get_note_frq(ch);
+	    ch->chPitch = ch->chFinalPitch = ch->chPortaToPitch = xm_player_get_note_pitch(ch);
 
 	    ch->nextpos=0;
+	    ch->sampleplayend=-1;
 
 	    if (proccmd==xmpCmdOffset) {
 		if (procdat!=0)
@@ -616,7 +623,7 @@ xm_player_playnote_fasttracker (channel *ch)
 	    ch->chMRetrigPos=0;
 	    ch->chTremorPos=0;
 	} else {
-	    ch->chPortaToPitch = xm_player_get_note_frq(ch);
+	    ch->chPortaToPitch = xm_player_get_note_pitch(ch);
 	}
     }
 
@@ -723,6 +730,9 @@ xmplayer_final_channel_ops (int chnr)
     }
     if(ch->nextpos != -1) {
 	driver_setsmplpos(chnr, ch->nextpos);
+	if(ch->sampleplayend != -1) {
+	    driver_setsmplend(chnr, ch->sampleplayend);
+	}
     }
     if(ismod) {
 	if(ch->chFinalPitch != 0) { /* == 0 happens on tru_funk.mod */
@@ -747,6 +757,13 @@ xmplayer_final_channel_ops (int chnr)
     } else {
 	driver_setpanning(chnr, (double)pan / 128);
     }
+
+    if(ch->chCutoff == 0xff && ch->chReso == 0) {
+	driver_set_ch_filter_freq(chnr, -1.0);
+    } else {
+	driver_set_ch_filter_freq(chnr, 0.5 * pow(2,(float)(ch->chCutoff-255)/32.0));
+	driver_set_ch_filter_reso(chnr, (float)ch->chReso / 255);
+    }
 }
 
 static gint32
@@ -795,7 +812,7 @@ static void xmpPlayTick()
     }
 
     curtick++;
-    if (curtick>=curtempo)
+    if (curtick>=player_tempo)
 	curtick=0;
 
     if(player_will_loop) {
@@ -803,7 +820,7 @@ static void xmpPlayTick()
 	player_will_loop = FALSE;
     }    
 
-    if (!curtick&&patdelay) {
+    if(!curtick && patdelay) {
 	if (jumptoord!=-1) {
 	    if (jumptoord!=curord)
 		for (i=0; i<nchan; i++) {
@@ -827,8 +844,8 @@ static void xmpPlayTick()
 	}
     }
 
-    if (!curtick && (!patdelay || ismod)) {
-	tick0=1;
+    if(!curtick && (!patdelay || ismod)) {
+	tick0 = 1;
 
 	if (!patdelay) {
 	    currow++;
@@ -859,6 +876,12 @@ static void xmpPlayTick()
 	    }
 	}
 
+	if(play_only_row != -1 && currow != play_only_row) {
+	    currow = play_only_row;
+	    xmplayer_playmode = PLAYING_NOTE;
+	    return;
+	}
+
 	for (i=0; i<nchan; i++) {
 	    channel *ch=&channels[i];
 
@@ -874,8 +897,6 @@ static void xmpPlayTick()
 	    }
 
 	    if (!patdelay) {
-		if (procnot==97)
-		    procins=0;
 		if (procins && procins<=ninst) {
 		    ch->chLastIns=ch->chCurIns;
 		    ch->chCurIns=procins;
@@ -965,6 +986,19 @@ static void xmpPlayTick()
 	    case xmpCmdPanning:
 		ch->chFinalPan=ch->chPan=procdat;
 		break;
+	    case xmpCmdSetFCutoff:
+		ch->chCutoff=procdat;
+		break;
+	    case xmpCmdSetFReso:
+		ch->chReso=procdat;
+		break;
+	    case xmpCmdSetFHFCutoff:
+//		mcpSet(0,mcpMasterFHFCutoff,procdat);
+		break;
+	    case xmpCmdSetFHFReso:
+//		mcpSet(0,mcpMasterFHFReso,procdat);
+		break;
+
 	    case xmpCmdJump:
 		if (!patdelay) {
 		    jumptoord=procdat;
@@ -990,17 +1024,17 @@ static void xmpPlayTick()
 		    break;
 		}
 		if (procdat>=0x20) {
-		    curbpm=procdat;
-		} else {
-		    curtempo=procdat;
-		}
+		    player_bpm = procdat;
+                } else {
+		    player_tempo = procdat;
+                }
 		break;
 	    case xmpCmdMODtTempo:
 		if (!procdat) {
 		    jumptoord=procdat;
 		    jumptorow=0;
 		} else {
-		    curtempo=procdat;
+		    player_tempo = procdat;
 		}
 		break;
 	    case xmpCmdGVolume:
@@ -1021,6 +1055,8 @@ static void xmpPlayTick()
 		break;
 	    case xmpCmdEnvPos:
 		ch->chVolEnvPos=ch->chPanEnvPos=procdat;
+		if(!ch->curins)
+		    break;
 		if (ch->curins->vol_env.flags & EF_ON)
 		    if (ch->chVolEnvPos > env_length(&ch->curins->vol_env))
 			ch->chVolEnvPos = env_length(&ch->curins->vol_env);
@@ -1038,8 +1074,6 @@ static void xmpPlayTick()
 		    ch->chMRetrigAct=procdat>>4;
 		}
 		break;
-	    case xmpCmdSync1: case xmpCmdSync2: case xmpCmdSync3:
-		break;
 	    case xmpCmdTremor:
 		if (procdat) {
 		    ch->chTremorLen=(procdat&0xF)+(procdat>>4)+2;
@@ -1314,10 +1348,9 @@ static void xmpPlayTick()
 	case xmpCmdMRetrigger:
 	    if (ch->chMRetrigPos++!=ch->chMRetrigLen)
 		break;
-	    ch->chMRetrigPos=1;
+	    ch->chMRetrigPos=0;
 	    ch->nextpos=0;
-	    ch->chVolEnvPos=0;
-	    ch->chPanEnvPos=0;
+	    ch->sampleplayend=-1;
 
 	    switch (ch->chMRetrigAct) {
 	    case 0: case 8: break;
@@ -1348,8 +1381,7 @@ static void xmpPlayTick()
 		break;
 	    if (!(curtick%ch->chActionTick)) {
 		ch->nextpos=0;
-		ch->chVolEnvPos=0;
-		ch->chPanEnvPos=0;
+		ch->sampleplayend=-1;
 	    }
 	    break;
 	case xmpCmdNoteCut:
@@ -1396,13 +1428,15 @@ xmplayer_init_module (void)
 {
     g_assert(xm != NULL);
 
-    curtempo = xm->tempo;
-    curbpm = xm->bpm;
+    player_tempo = xm->tempo;
+    player_bpm = xm->bpm;
 }
 
 static gboolean
 xmplayer_init_playing ()
 {
+    int i;
+
     memset(channels, 0, sizeof(channels));
 
     driver_setnumch(xm->num_channels);
@@ -1418,18 +1452,36 @@ xmplayer_init_playing ()
     linearfreq = !(xm->flags & XM_FLAGS_AMIGA_FREQ);
     nchan = xm->num_channels;
     loopord = xm->restart_position;
-    curtick = curtempo-1;
+    curtick = player_tempo-1;
     patdelay = 0;
 
+    for(i = 0; i < nchan; i++) {
+	channels[i].chCutoff = 0xff;
+	channels[i].chReso = 0;
+    }
+
     return TRUE;
 }
 
+void
+xmplayer_set_tempo (int tempo)
+{
+    player_tempo = tempo;
+}
+
+void
+xmplayer_set_bpm (int bpm)
+{
+    player_bpm = bpm;
+}
+
 gboolean
 xmplayer_init_play_song (int songpos,
 			 int patpos)
 {
     jumptorow = patpos;
     currow = patpos;
+    play_only_row = -1;
     jumptoord = songpos;
     curord = songpos;
     player_will_loop = FALSE;
@@ -1444,10 +1496,12 @@ xmplayer_init_play_song (int songpos,
 
 gboolean
 xmplayer_init_play_pattern (int pattern,
-			    int patpos)
+			    int patpos,
+			    int only1row)
 {
     jumptorow = patpos;
     currow = patpos;
+    play_only_row = only1row ? patpos : -1;
     jumptoord = 0;
     curord = 0;
     patlen = xm->patterns[pattern].length;
@@ -1485,8 +1539,12 @@ xmplayer_play_note (int channel,
 
     PlayNote(channel);
 
+    channels[channel].chCutoff = 0xff;
+    channels[channel].chReso = 0;
+
     channels[channel].nextpos = -1;
-    channels[channel].nextstop = 0;
+    channels[channel].sampleplayend = -1;
+    channels[channel].nextstop = 1;
     xmplayer_final_channel_ops(channel);
 
     return TRUE;
@@ -1496,7 +1554,8 @@ gboolean
 xmplayer_play_note_full (int chnr,
 			 int note,
 			 STSample *sample,
-			 int offset)
+			 guint32 offset,
+			 guint32 count)
 {
     gint32 nn;
     channel *ch = &channels[chnr];
@@ -1522,15 +1581,13 @@ xmplayer_play_note_full (int chnr,
     ch->chDefVol = ch->cursamp->volume;
     ch->chDefPan = ch->cursamp->panning;
 
-    ch->nextstop=1;
-
     ch->nextsamp = ch->cursamp;
 
     /* CurNormNote is only relevant in FastTracker mode */
     nn = -ch->cursamp->relnote*256 - ch->cursamp->finetune*2;
     ch->chCurNormNote = nn;
     
-    ch->chPitch = ch->chFinalPitch = ch->chPortaToPitch = xm_player_get_note_frq(ch);
+    ch->chPitch = ch->chFinalPitch = ch->chPortaToPitch = xm_player_get_note_pitch(ch);
 	    
     ch->chVibPos=0;
     ch->chTremPos=0;
@@ -1550,11 +1607,14 @@ xmplayer_play_note_full (int chnr,
     ch->chAVibSwpPos=0;
     ch->chVolEnvPos=0;
     ch->chPanEnvPos=0;
+    ch->chCutoff = 0xff;
+    ch->chReso = 0;
 
     ch->nextsamp = sample;
     ch->hacksample = 1;
+    ch->nextstop = 1;
     ch->nextpos = offset;
-    ch->nextstop = 0;
+    ch->sampleplayend = offset + count;
     xmplayer_final_channel_ops(chnr);
 
     return TRUE;
@@ -1573,7 +1633,8 @@ xmplayer_play (void)
     player_patpos = currow;
 
     xmpPlayTick();
-    current_time += (double)125 / (curbpm * 50);
+
+    current_time += (double)125 / (player_bpm * 50);
     return current_time;
 }
 
@@ -1582,7 +1643,7 @@ xmplayer_set_songpos (int songpos)
 {
     jumptorow = currow = player_patpos = 0;
     jumptoord = curord = player_songpos = songpos;
-    curtick = curtempo - 1;
+    curtick = player_tempo - 1;
     globalvol = 64;
 }
 
