@@ -60,6 +60,10 @@ static guint16 npertab[60]={
     107, 101,  95,  90,  85,  80,  75,  71,  67,  63,  60,  56
 };
 
+/* This function calculates relnote and finetune settings such that
+   the sample, when played back at XM note 'note' (for example, C-6)
+   will be played back at sample frequency 'frequency'. */
+
 void
 xm_freq_note_to_relnote_finetune (float frequency,
 				  unsigned note,
@@ -67,27 +71,24 @@ xm_freq_note_to_relnote_finetune (float frequency,
 				  gint8 *finetune)
 {
     int pitch = - log(frequency / 8363.0) / M_LN2 * (float)PITCH_OCTAVE;
-    int nn = 256 * (49 - note) - pitch;
+    int nn = 128 * (49 - note) - pitch / 2;
     int rn;
 
-    /* nn is now 256*relnote + 2*finetune. Find those two unknowns. */
-    for(rn = 0; nn < -256; rn--) {
-	nn += 256;
-    }
-    for(; nn > 255; rn++) {
-	nn -= 256;
+    /* Now, calculate relnote and finetune such that nn = (128*relnote
+       + finetune).  The XM format allows a finetune in the interval
+       [-128, +127].  Because of this, we typically have two solutions
+       to the above equation. We use the one with finetune in the
+       interval [-64, +63]. */
+
+    if(nn < 0) {
+        rn = (nn - 63) / 128;
+    } else {
+        rn = (nn + 64) / 128;
     }
 
-    if(rn < -96) {
-	rn = -96;
-    } else if(rn > +96) {
-	rn = +96;
-    }
-
-    g_assert(nn >= -256 && nn <= 255);
-
+    *finetune = nn - (rn * 128);
+    rn = CLAMP(rn, -128, +127);
     *relnote = rn;
-    *finetune = nn / 2;
 }
 
 static void
@@ -184,6 +185,8 @@ xm_load_xm_pattern (XMPattern *pat,
     guint8 ph[9];
     int i, j;
     guint16 len;
+    guint32 hdr_len;
+    unsigned long new_position, position;
 
     fread(ph, 1, sizeof(ph), f);
 
@@ -195,6 +198,11 @@ xm_load_xm_pattern (XMPattern *pat,
 	return 0;
     }
 
+    /* skip the rest of the header, if exists */
+    if ((hdr_len = get_le_32(ph)) > 9)
+	fseek(f, hdr_len - 9, SEEK_CUR);
+    position = ftell(f);
+
     if(!st_init_pattern_channels(pat,
 				 len > 0 ? len : 1,
 				 (num_channels + 1) & 0xfe))
@@ -210,6 +218,11 @@ xm_load_xm_pattern (XMPattern *pat,
 	}
     }
 
+    /* skip the rest of patterndata, which is really waste, but can exist */
+    new_position = ftell(f);
+    if((new_position - position) < get_le_16(ph + 7))
+	fseek(f, get_le_16(ph + 7) - new_position + position, SEEK_CUR);
+
     return 1;
 }
 
@@ -440,7 +453,7 @@ static int
 xm_load_xm_instrument (STInstrument *instr,
 		       FILE *f)
 {
-    guint8 a[29], b[38];
+    guint8 a[29], b[16];
     guint16 num_samples;
     guint32 iheader_size;
 
@@ -462,12 +475,8 @@ xm_load_xm_instrument (STInstrument *instr,
     }
 
     if(num_samples == 0) {
-	int to_read = iheader_size - sizeof(a);
-	while(to_read) {
-	    int r = MIN(to_read, sizeof(a));
-	    fread(a, 1, r, f);
-	    to_read -= r;
-	}
+	/* Skip rest of header */
+	fseek(f, iheader_size - sizeof(a), SEEK_CUR);
     } else {
 	fread(a, 1, 4, f);
 	if(get_le_32(a) != 40) {
@@ -508,6 +517,11 @@ xm_load_xm_instrument (STInstrument *instr,
 	
 	instr->volfade = get_le_16(b + 14);
 
+	if(iheader_size > 241) {
+            /* Skip remainder of header */
+	    fseek(f, iheader_size - 241, SEEK_CUR);
+	}
+
 	xm_load_xm_samples(instr->samples, num_samples, f);
     }
 
@@ -950,6 +964,11 @@ XM_Load (const char *filename,int *status)
     }
 
     xm->num_channels = get_le_16(xh + 68);
+    if(xm->num_channels > 32 || xm->num_channels < 1) {
+	error_error("Invalid number of channels in XM (only 1..32 allowed).");
+	goto ende;
+    }
+
     num_patterns = get_le_16(xh + 70);
     num_instruments = get_le_16(xh + 72);
     if(get_le_16(xh + 74) != 1) {
@@ -1384,3 +1403,95 @@ File_Load (const char *filename)
 
     return ret;
 }
+
+gboolean
+xm_xp_load_header (FILE *f, int *length)
+{
+    guint8 pheader[4];
+    int version;
+    
+    if ( fread (pheader, 1, sizeof(pheader), f) != 4) {
+	error_error (_("Error when file reading or unexpected end of file"));
+	return FALSE;
+    }
+    if ((version = pheader[0] + (pheader[1] << 8)) != 1) {
+	error_error (_("Incorrect or unsupported version of pattern file!"));
+	return FALSE;
+    }
+    if (((*length = pheader[2] + (pheader[3] << 8)) < 0) || (*length > 256)){
+	error_error (_("Incorrect pattern length!"));
+	return FALSE;
+    }
+    return TRUE;
+}
+
+gboolean
+xm_xp_load (FILE *f, int length, XMPattern *patt, XM *xm)
+{
+    static guint8 buf[32*256*5];
+    int i, j, bp;
+    
+    if (fread (buf, 1, length * 32 * 5, f) != length*32*5) {
+	error_error (_("Error when file reading or unexpected end of file"));
+	return FALSE;
+    }
+    for (j = 0; j <= MIN (length, patt->length) - 1; j++) {
+	bp = j * 5 * 32;
+	for (i = 0; i <= xm->num_channels - 1; i++) {
+	    memcpy (&patt->channels[i][j].note, &buf[bp], 5);
+	    bp += 5;
+	}
+    }
+    if (length < patt->length)
+	for (j = length; j < patt->length; j++) {
+	    bp = j * 5 * 32;
+	    for (i = 0; i <= xm->num_channels - 1; i++) {
+		memset (&patt->channels[i][j].note, 0, 5);
+		bp += 5;
+	    }
+	}
+    return TRUE;
+}
+
+void
+xm_xp_save (gint reply, gpointer data)
+{
+    struct f_n_p *fnp = (struct f_n_p*) data;
+
+    FILE *f;
+    int i, j, bp;
+    guint8 pheader[4];
+    static guint8 buf[32*256*5];
+    
+    if (reply == 0){
+	if ( !(f = fopen (fnp->name, "wb"))) 
+	    error_error (_("Error during saving pattern!"));
+	else {
+	    put_le_16 (pheader+0, 01);//version
+	    put_le_16 (pheader+2, fnp->pattern->length);//length
+	    
+	    bp = 0;
+	    for (j = 0; j <= fnp->pattern->length - 1; j++)//row
+		for (i = 0; i <= 31; i++){//ch
+		    if ( i <= fnp->xm->num_channels - 1){
+			buf[bp + 0] = fnp->pattern->channels[i][j].note;
+			buf[bp + 1] = fnp->pattern->channels[i][j].instrument;
+			buf[bp + 2] = fnp->pattern->channels[i][j].volume;
+			buf[bp + 3] = fnp->pattern->channels[i][j].fxtype;
+			buf[bp + 4] = fnp->pattern->channels[i][j].fxparam;
+		    } else {
+			buf[bp + 0] = 0;
+			buf[bp + 1] = 0;
+			buf[bp + 2] = 0;
+			buf[bp + 3] = 0;
+			buf[bp + 4] = 0;
+		    }
+		    bp += 5;
+		}
+	    
+	    fwrite (pheader, 1, sizeof(pheader), f);
+	    fwrite (buf, 1, bp, f);
+	    fclose (f);
+	}
+    }
+}
